-- $Id: farmer-maggot.lua 2941 2017-06-20 02:16:32Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local q = game.player:hasQuest('mushroom-farming')
local person = game.player.descriptor.sex == 'Female' and 'lass' or 'fella'

local function num_mushrooms(player)
  local n = 0
  local inven = player:getInven('INVEN')
  for _, o in ipairs(inven) do
    if o.quest_obj == 'mushroom-farming' then n = n + o:getNumber() end
  end
  return n
end

local function take_mushrooms(player)
  repeat
    local o, item, inven = player:findInAllInventoriesBy('quest_obj', 'mushroom-farming')
    if o then player:removeObject(inven, item, true) end
  until not o
end

newChat {
  id = 'welcome',
  text = [[Hallo there, young ]]..person..[[.  Can I interest you in some mushroom farming?]],
  answers = {
    { 'Mushroom farming?', jump='give_quest' },
    { 'Uh... no? [leave]' },
    { '#{italic}#*blink*#{normal}# ...O-kaay, is that a euphemism for something?', jump='give_quest' },
  },
}

newChat {
  id = 'give_quest',
  text = [[Well, y'see, my crop's come in, but my dogs have kinda run me off.  Dunno what's got into `em, but they're mighty agitated.  I don't dare venture into the fields with them in this state, but I reckon they can't scare off a strong ]]..person..[[ like you.  I do insist, though, that you not hurt my dogs while you're at it.]],
  answers = {
    {
      [[#{italic}#(shrug)#{normal}# Okay, I'll see what I can do.]],
      action = function(npc, player) player:grantQuest('mushroom-farming') end,
    },
    { "Yyyeah, y'know what?  Changed my mind.  I'm out.  [leave]" },
  },
}

newChat {
  id = 'welcome_quest',
  text = [[Hallo, @playername@.  How goes the mushroom farming?]],
  answers = {
    {
      [[Right, I did say I was going to do that, didn't I?]],
      cond = function(npc, player) return num_mushrooms(player) == 0 end,
    },
    {
      [[I've got some of your mushrooms.]],
      cond = function(npc, player)
	local n = num_mushrooms(player)
	return n > 0 and n + q.mushrooms_returned < q.total_mushrooms
      end,
      action = function(npc, player)
	take_mushrooms(player)
      end,
      jump = 'give_some_mushrooms',
    },
    {
      [[Okay, I think this is the last of `em.]],
      cond = function(npc, player)
	local n = num_mushrooms(player)
	return n + q.mushrooms_returned >= q.total_mushrooms
      end,
      action = function(npc, player)
	take_mushrooms(player)
      end,
      jump = 'give_all_mushrooms',
    },
  },
}

newChat {
  id = 'give_some_mushrooms',
  text = [[Ah, good, good.  I hope the dogs weren't too much trouble.  Good luck with the rest of the crop.]],
  answers = {
    { 'Right.' },
  },
}

newChat {
  id = 'give_all_mushrooms',
  text = [[Ah, wonderful!  Maybe the dogs'll calm down now.  Here, let me give you something in thanks.  Most of the young'uns `round these parts know better than to come around mushroom-thievin' by now, so I don't use it all that much; I bet an adventurous ]]..person..[[ like yourself could put it to good use.]],
  answers = {
    {
      [[Thanks.]],
      action = function(npc, player)
	local o = game.zone:makeEntityByName(game.level, 'object', 'OF_FARMER_MAGGOT', true)
	if o then
	  o:identify('full', player)
	  game.zone:addEntity(game.level, o, 'object')
	  player:addOrDropObject(player.INVEN_INVEN, o, nil, nil, {vocal_add=true, vocal_drop=true})
	  player:sortInven()
	end
	if game.level:hasEntity(npc) then game.level:removeEntity(npc) end
	player:setQuestStatus('mushroom-farming', engine.Quest.DONE)
      end,
    },
  },
}

newChat {
  id = 'welcome_quest',
  text = [[#{italic}#TODO#{normal}#]],
  answers = {
    {
      [[I'm sorry.]],
      action = function(npc, player)
	if game.level:hasEntity(npc) then game.level:removeEntity(npc) end
      end,
    },
  },
}

return not q and 'welcome' or q:isStatus(q.FAILED) and 'welcome_fail' or 'welcome_quest'
