-- $Id: galadriel.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

newChat {
  id = 'welcome',
  text = [[#LIGHT_GREEN#*You find Galadriel standing before a wide and shallow silver basin filled with water, set on a low pedestal.*#LAST#
Ah, hello, @playername@.  You have found me at my Mirror.  How can I be of assistance to you?]],
  answers = {
    {
      'Milady, I sense a foreboding in my future.  Can your Mirror tell me my fate? #LIGHT_BLUE#(500gp)#LAST#',
      cond = function(npc, player)
	return player:hasUnknowFates() and player.gold >= 500
      end,
      action = function(npc, player, dlg)
	player.gold = player.gold - 500
	player.changed = true
	local fate_desc = player:learnUnknownFate()
	if fate_desc then
	  -- Deep wizardry:  we reach ahead into the next chat object and
	  -- replace its text and answer with ours.
	  local chat = dlg.chat:get('fate')
	  chat.text = [[#LIGHT_GREEN#*Galadriel peers into the Mirror.*#LAST#  Hmm... yes, I can see an image.  It seems ]] .. fate_desc:uncapitalize()
	end
	return nil
      end,
      jump = 'fate'
    },
    {
      'Milady, I have found this item.  Can your Mirror tell me anything about it? #LIGHT_BLUE#(1500gp)#LAST#',
      cond = function(npc, player)
	local function cb(o, _, _) return not o:isIDd(true) end
	return player.gold >= 1500 and util.findInAllInventoriesBy(player, cb)
      end,
      action = function(npc, player)
	local function filter_cb(o) return not o:isIDd(true) end
	local function action_cb(o, _, _, _, _, _)
	  if o then
	    player.gold = player.gold - 1500
	    player.changed = true
	    o:identify('full', player)
	    local Dialog = require 'engine.ui.Dialog'
	    Dialog:simpleLongPopup(o:getName(), o:getDesc(), 600, nil, false, 0.8)
	    player:sortInven()
	    if player.recalcEverything then player:recalcEverything() end
	  end
	  return true
	end
	local ShowEquipInvenFloor = require 'mod.dialogs.ShowEquipInvenFloor'
	game:registerDialog(ShowEquipInvenFloor.new('Select item to fully identify', self, filter_cb, action_cb))
      end,
    },
    { "Nothing, Milady.  Forgive me for disturbing you. [leave]" },
  },
}

newChat {
  id = 'fate',
  text = [[#LIGHT_GREEN#*Galadriel peers into the Mirror.*#LAST#  Hmm, it seems there is an error in the program.  Your fate should be here.  Odd.]],
  answers = {
    { "Thank you, Milady. [leave]" },
  },
}

newChat {
  id = 'necromancer1',
  text = [[#LIGHT_GREEN#*You find Galadriel standing at her Mirror.  She seems worried.#LAST#
Ah, @playername@.  Rumors from Dol Guldur precede you.  Is it true that you have slain the Necromancer?]],
  answers = {
    {
      [[I have, Milady, and I have learned that he was in fact Sauron in disguise.]],
      action = function(npc, player)
	player:setQuestStatus('necromancer', engine.Quest.DONE)
	return nil
      end,
      jump='necromancer2',
    }
  },
}

newChat {
  id = 'necromancer2',
  text = [[It is as we feared, then.  This is bad.  The Enemy is growing in power; but while Sauron lives, we cannot hope to reach Morgoth to fight back.]],
  answers = {
    { 'Then we take the fight to Sauron?', jump='necromancer3' },
  },
}

newChat {
  id = 'necromancer3',
  text = [[No, that would be futile while the One Ring still exists.  Sauron's power is bound to it, and while it endures, he cannot be permanently slain.  Our only hope is to find the Ring and destroy it.]],
  answers = {
    {
      'How can we do that?',
      action = function(npc, player, dlg)
	-- Slight Hack(TM):  Add a Melkor mention to the next chat if we're
	-- following him.
	if player:isFollowing('Melkor') then
	  local chat = dlg.chat:get('necromancer4')
	  chat.text = chat.text .. [[  You are a follower of @playerdescriptor.god@, are you not?  He will no doubt abandon you for the affront, but it must be done.]]
	end
	return nil
      end,
      jump='necromancer4'
    },
  },
}

newChat {
  id = 'necromancer4',
  text = [[As for finding it, we have been studying the history of the Ring, and I can suggest a few places where it might have fallen; most likely, though, you will have to take it from Sauron himself.  To destroy it, you must take it to Mount Doom in Mordor and drop it into the Great Fire in which it was forged.]],
  answers = {
    {
      'That sounds perilous.',
      action = function(npc, player, dlg)
	-- Slight Hack(TM):  Add a Vala mention to the next chat if we're
	-- following one (except for Melkor, which we handled above).
	if not player:isFollowing('Melkor') and not player:isFollowing('Nobody') then
	  -- Dodgy Hack(TM):  Insert a fake descriptor for chat text
	  -- convenience.
	  player.descriptor.fake_god_Pronoun = player:isFollowing('Yavanna') and 'She' or 'He'
	  local chat = dlg.chat:get('necromancer5')
	  chat.text = chat.text .. [[  You are a follower of @playerdescriptor.god@, are you not?  @playerdescriptor.god_Pronoun@ will abandon you if you wear the Ring.]]
	end
      end,
      jump='necromancer5'
    },
  },
}

newChat {
  id = 'necromancer5',
  text = [[And there is one more thing you must remember.  The Ring can have a powerful influence on the mind of its bearer.  It will tempt you to wield it.  You must #{italic}#never#{normal}# do so; its power would corrupt you forever.]],
  answers = {
    {
      'I understand.',
      action = function(npc, player)
	-- Dodgy Hack(TM), redux:  Clean this up behind ourself.
	player.descriptor.fake_god_Pronoun = nil
	player:grantQuest('one-ring')
      end,
    }
  },
}

if game.player:isQuestStatus('necromancer', engine.Quest.COMPLETED) then
  return 'necromancer1'
end
return 'welcome'
