-- $Id: prancing-pony.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Dialog = require 'engine.ui.Dialog'

local rumor_pfx = {
  [[The bartender offers you some advice:]],
  [[You overhear someone at the bar say:]],
  [[One of the tables looks rowdy; someone shouts:]],
}

-- TODO:
--   - Actually implement the unimplemented actions.
--   - Handle pricing for different races.
--   - Change as necessary for vampires.
newChat {
  id = 'welcome',
  text = [[Welcome to the Prancing Pony.  Barliman Butterbur at your service.  Rooms are 20 gold a night, supper is 2 gold, and the bar's open till eleven.  And for the adventurer type (and you have the look of one if I may say so, beggin' your pardon), the shop sells food and drink for traveling.  Now, what can I do for you?]],
  answers = {
    {
      'A room for the night, please.',
      action = function(npc, player)
	Dialog:simplePopup('Not implemented', 'This feature is not yet implemented.  We Apologize for the Inconvenience.™')
      end,
    },
    {
      'A meal, please.',
      action = function(npc, player)
	-- TODO Handle vampires.
	game.log('The barkeep gives you some gruel and a beer.')
	player.gold = player.gold - 2
	player.changed = true
	player:setFood(util.FOOD.max - 1)
      end,
      cond = function(npc, player) return player.gold >= 2 end,
    },
    {
      'You mentioned a shop?',
      action = function(npc, player)
	npc.store:loadup(game.level, game.zone)
	npc.store:interact(player)
      end
    },
    {
      "I think I'll go the bar and listen for rumors.",
      action = function(npc, player, dlg)
	local f, err = loadfile('/data/rumors.lua')
	local text
	if f then
	  local rumor = f()
	  local pfx = rumor_pfx[rng.range(1, #rumor_pfx)]
	  text = pfx..'\n\n'..rumor
	else
	  print('[RUMOR] failed to load rumors.lua', err)
	  text = 'The bar seems quiet; not much to be heard.'
	end
	-- Deep wizardry:  we reach ahead into the next chat object and
	-- replace its text with ours.
	dlg.chat:get('rumor').text = text
	-- I think this makes us fall through to the jump.
	return nil
      end,
      jump = 'rumor'
    },
    { "Oh, this isn't the magic shop?  Sorry.  [leave]" },
  },
}

newChat {
  id = 'rumor',
  text = '??? This text should have been replaced by now',
  answers = {
    { 'Ohh...kay, then. [leave]' }
  },
}

return 'welcome'
