-- $Id: mimicry_forms.lua 2875 2017-05-11 01:23:13Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local function num_info(...)
  local args = {...}
  local desc = tstring{}
  for i = 1, #args, 2 do
    local f, v = args[i], args[i+1]
    if i > 1 then desc:add(', ') end
    desc:merge(util.colorize_num(v):toTString())
    desc:add(' '..f)
  end
  return desc
end

local function list_info(nl, lev, ...)
  local args = {...}
  local ff = {}
  for i = 1, #args, 2 do
    local l, f = args[i], args[i+1]
    if lev >= l then table.insert(ff, f) end
  end
  if #ff == 0 then return tstring{} end
  ff = table.concat(ff, ', ')
  return nl and tstring{true, ff:capitalize()} or tstring{ff}
end

local function resist_info(pfx, nl, lev, ...)
  local DamageType = require 'engine.DamageType'
  local args = {...}
  local desc = nl and tstring{ true, pfx..' '} or tstring{ pfx..' ' }
  local any = false
  for i = 1, #args, 2 do
    local l, id = args[i], args[i+1]
    if lev >= l then
      local dt = DamageType:get(id)
      if any then desc:add(', ') end
      desc:merge(util.colorize_damtype(id, true))
      any = true
    end
  end
  return any and desc or tstring{}
end

newMimicryForm {
  name = 'Mouse',
  cloak_name = '& Mouse Fur~',
  desc = 'Mice are small, fast and very stealthy',
  realm = 'nature',
  level = 1,
  rarity = 10,
  duration = { 20, 40 },
  eff_id = 'MOUSE_FORM',
  apply = function(who, lev)
    who.speed = who.speed + 5 + math.floor(lev/7)
    who:adjustDis('to_h', 10 + math.floor(lev/5), true, 'Mouse form')
    who.to_d = math.floor(who.to_d/5)
    who.dis_to_d = math.floor(who.dis_to_d/5)
    who.skill_stealth = who.skill_stealth + 10 + math.floor(lev/5)
    who:adjustStatForEquip('STR', -5)
    who:adjustStatForEquip('DEX', 3)
    who:adjustStatForEquip('CON', 1)
    who:accumTargetTalent('T_CLOAK_MOUSE_INVIS')
  end,
  info = function(who, lev)
    local desc = num_info('STR', -5, 'DEX', 3, 'CON', 1)
    desc:add(true)
    desc:merge(num_info('to hit', 10 + math.floor(lev/5),
			'to damage', math.floor(who.to_d/5) - who.to_d))
    desc:add(true)
    desc:merge(num_info('speed', 5 + math.floor(lev/7),
			'stealth', 10 + math.floor(lev/5)))
    desc:add(true, 'Invisibility power')
    return desc
  end,
}

newMimicryForm {
  name = 'Eagle',
  cloak_name = '& Feathers Cloak~',
  desc = 'Eagles are master of the air, good hunters with excellent vision',
  realm = 'nature',
  level = 10,
  rarity = 30,
  duration = { 10, 50 },
  apply = function(who, lev)
    who.flags.FEATHER = true
    who.speed = who.speed + 2 + math.floor(lev/6)
    who:adjustStatForEquip('STR', -3)
    who:adjustStatForEquip('DEX', 2 + math.floor(lev/15))
    who:adjustStatForEquip('CON', 4 + math.floor(lev/20))
    who:adjustStatForEquip('INT', -1)
    who:adjustStatForEquip('WIS', 1)
    who:adjustStatForEquip('CHA', -1)
    if lev >= 20 then
      who.flags.FLY = true
      who.flags.SEE_INVIS = true
    end
    if lev >= 25 then
      who.flags.FREE_ACT = true
    end
    if lev >= 30 then
      local DamageType = require 'engine.DamageType'
      who.resist_dam[DamageType.ELEC] = true
      who.flags.SH_ELEC = true
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', -3, 'INT', -1, 'WIS', 1,
			  'DEX', 2 + math.floor(lev/15),
			  'CON', 4 + math.floor(lev/20),
			  'CHA', -1)
    desc:add(true)
    desc:merge(num_info('speed', 2 + math.floor(lev/6)))
    desc:merge(resist_info('Resist', true, lev, 30, DamageType.ELEC))
    desc:merge(list_info(true, lev, 20, 'flight', 20, 'see invisible',
				    25, 'free action', 30, 'lightning aura'))
    return desc
  end,
}

newMimicryForm {
  name = 'Wolf',
  cloak_name = '& Wolf Pelt~',
  desc = 'Wolves are masters of movement, strong and have excellent eyesight',
  ream = 'nature',
  level = 20,
  rarity = 40,
  duration = { 10, 50 },
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who:adjustStatForEquip('STR', 2 + math.floor(lev/20))
    who:adjustStatForEquip('DEX', 3 + math.floor(lev/20))
    who:adjustStatForEquip('INT', -3)
    who:adjustStatForEquip('CHA', -2)
    who.speed = who.speed + 10 + math.floor(lev/5)
    who.flags.FREE_ACT = true
    who.resist_dam[DamageType.FEAR] = true
    if lev >= 10 then
      who.resist_dam[DamageType.COLD] = true
    end
    if lev >= 15 then
      who.flags.SEE_INVIS = true
    end
    if lev >= 30 then
      who.resist_dam[DamageType.DARK] = true
    end
    if lev >= 35 then
      who.resist_dam[DamageType.CONFUSION] = true
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', 2 + math.floor(lev/20),
			  'INT', -3,
			  'DEX', 3 + math.floor(lev/20),
			  'CHA', -2)
    desc:add(true)
    desc:merge(num_info('speed', 10 + math.floor(lev/5)))
    desc:merge(resist_info('Resist', true, lev,
			   10, DamageType.COLD,
			   30, DamageType.DARK,
			   35, DamageType.CONFUSION))
    desc:merge(list_info(true, lev, 15, 'see invisible'))
    return desc
  end,
}

newMimicryForm {
  name = 'Spider',
  cloak_name = '& Spider Web~',
  desc = 'Spiders are clever and become good climbers',
  realm = 'nature',
  level = 25,
  rarity = 50,
  duration = { 10, 50 },
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who:adjustStatForEquip('STR', -4)
    who:adjustStatForEquip('DEX', 1 + math.floor(lev/8))
    who:adjustStatForEquip('INT', 1 + math.floor(lev/5))
    who:adjustStatForEquip('WIS', 1 + math.floor(lev/5))
    who:adjustStatForEquip('CON', -5)
    who:adjustStatForEquip('CHA', -10)
    who.speed = who.speed + 5
    who.resist_dam[DamageType.POISON] = true
    who.resist_dam[DamageType.FEAR] = true
    who.resist_dam[DamageType.DARK] = true
    who.flags.SPIDER = true
    if lev >= 40 then
      who.flags.CLIMB = true
    end
    if lev >= 25 then
      who:accumTargetTalent('T_CLOAK_SPIDER_WEB')
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', -4,
			  'INT', 1 + math.floor(lev/5),
			  'WIS', 1 + math.floor(lev/5),
			  'DEX', 1 + math.floor(lev/8),
			  'CON', -5,
			  'CHA', -10)
    desc:add(true)
    desc:merge(num_info('speed', 5))
    desc:merge(resist_info('Resist', true, lev,
			   0, DamageType.FEAR, 0, DamageType.DARK))
    desc:merge(list_info(true, lev, 40, 'mountain climbing', 20, 'web power',
				     0, 'can traverse webs'))
    return desc
  end,
}

newMimicryForm {
  name = 'Elder Ent',
  cloak_name = '& Entish Bark~',
  desc = 'Ents are powerful tree-like beings dating from the dawn of time',
  ream = 'nature',
  level = 40,
  rarity = 60,
  duration = { 10, 30 },
  no_potion = true,
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who.speed = who.speed - 5 - math.floor(lev/10)
    who:adjustDis('to_a', 10 + lev, true)
    who:adjustStatForEquip('STR', math.floor(lev/5))
    who:adjustStatForEquip('INT', -math.floor(lev/7))
    who:adjustStatForEquip('WIS', -math.floor(lev/7))
    who:adjustStatForEquip('DEX', -4)
    who:adjustStatForEquip('CON', math.floor(lev/5))
    who:adjustStatForEquip('CHA', -7)
    who.resist_dam[DamageType.POISON] = true
    who.resist_dam[DamageType.COLD] = true
    who.flags.FREE_ACT = true
    who.flags.REGENERATE = true
    who.flags.SEE_INVIS = true
    who.flags.PASS_TREE = true
    who.vuln_dam[DamageType.FIRE] = true
    who:accumTargetTalent('T_ENT_GROW_TREES')
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', math.floor(lev/5),
			  'INT', -math.floor(lev/7),
			  'WIS', -math.floor(lev/7),
			  'DEX', -4,
			  'CON', math.floor(lev/5),
			  'CHA', -7)
    desc:add(true)
    desc:merge(num_info('AC', 10 + lev, 'speed', -5 - math.floor(lev/10)))
    desc:merge(resist_info('Resist', true, lev,
			   0, DamageType.POISON, 0, DamageType.COLD))
    desc:merge(resist_info('Vulnerable to', true, lev, 0, DamageType.FIRE))
    desc:merge(list_info(true, lev, 0, 'free action', 0, 'regeneration',
				    0, 'see invisible',
				    0, 'can traverse trees'))
    desc:merge(list_info(true, lev, 0, 'grow-tree power'))
    return desc
  end,
}

newMimicryForm {
  name = 'Vapour',
  cloak_name = '& Cloak~ of Mist',
  desc = 'A sentient cloud, darting around',
  realm = 'nature',
  level = 15,
  rarity = 10,
  duration = { 10, 40 },
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who.speed = who.speed + 5
    who:adjustDis('to_a', 40 + lev, true)
    who:adjustDis('to_h', -40, true, 'Vapour form')
    who:adjustStatForEquip('STR', -4)
    who:adjustStatForEquip('DEX', 5)
    who:adjustStatForEquip('CON', -4)
    who:adjustStatForEquip('CHA', -10)
    who.skill_stealth = who.skill_stealth + 10 + math.floor(lev/5)
    who.resist_dam[DamageType.POISON] = true
    who.resist_dam[DamageType.SHARDS] = true
    who.immune_dam[DamageType.COLD] = true
    who.vuln_dam[DamageType.FIRE] = true
    who.flags.FREE_ACT = true
    who.flags.REGENERATE = true
    who.flags.SEE_INVIS = true
    who.flags.FEATHER = true
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', -4, 'DEX', 5, 'CON', -4, 'CHA', -10)
    desc:add(true)
    desc:merge(num_info('AC', 40 + lev, 'to hit', -40, 'speed', 5,
			'stealth', 10 + math.floor(lev/5)))
    desc:merge(resist_info('Resist', true, lev,
			   0, DamageType.POISON, 0, DamageType.SHARDS,
			   0, DamageType.COLD))
    desc:merge(resist_info('Vulnerable to', true, lev, 0, DamageType.FIRE))
    desc:merge(list_info(true, lev, 0, 'free action', 0, 'regeneration',
				    0, 'see invisible', 0, 'feather fall'))
    return desc
  end,
}

newMimicryForm {
  name = 'Serpent',
  cloak_name = '& Snakeskin Cloak~',
  desc = 'Serpents are fast, lethal predators',
  realm = 'nature',
  level = 30,
  rarity = 25,
  duration = { 15, 20 },
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who.speed = who.speed + 10 + math.floor(lev/6)
    who:adjustDis('to_a', 3 + math.floor(lev/8), true)
    who:adjustStatForEquip('STR', math.floor(lev/8))
    who:adjustStatForEquip('INT', -6)
    who:adjustStatForEquip('WIS', -6)
    who:adjustStatForEquip('DEX', -4)
    who:adjustStatForEquip('CON', math.floor(lev/7))
    who:adjustStatForEquip('CHA', -6)
    who.resist_dam[DamageType.POISON] = true
    if lev >= 25 then
      who.flags.FREE_ACT = true
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', math.floor(lev/8), 'INT', -6, 'WIS', -6,
			  'DEX', -4, 'CON', math.floor(lev/7), 'CHA', -6)
    desc:add(true)
    desc:merge(num_info('AC', 3 + math.floor(lev/8),
			'speed', 10 + math.floor(lev/6)))
    desc:merge(resist_info('Resist', true, lev, 0, DamageType.POISON))
    desc:merge(list_info(true, lev, 25, 'free action'))
    return desc
  end,
}

newMimicryForm {
  name = 'Mumak',
  cloak_name = '& Mumak Hide~',
  desc = 'A giant, elephantine form',
  realm = 'nature',
  level = 40,
  rarity = 40,
  duration = { 15, 20 },
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who.speed = who.speed - 5 - math.floor(lev/10)
    who:adjustDis('to_a', 10 + math.floor(lev/6), true)
    who:adjustDis('to_d', 5 + math.floor((lev*2)/3), true)
    who:adjustStatForEquip('STR', math.floor(lev/4))
    who:adjustStatForEquip('INT', -8)
    who:adjustStatForEquip('WIS', -4)
    who:adjustStatForEquip('DEX', -5)
    who:adjustStatForEquip('CON', math.floor(lev/3))
    who:adjustStatForEquip('CHA', -10)
    if lev >= 10 then
      who.resist_dam[DamageType.FEAR] = true
    end
    if lev >= 25 then
      who.resist_dam[DamageType.CONFUSION] = true
    end
    if lev >= 30 then
      who.flags.FREE_ACT = true
    end
    if lev >= 35 then
      who.resist_dam[DamageType.NEXUS] = true
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', math.floor(lev/4), 'INT', -8, 'WIS', -4,
			  'DEX', -5, 'CON', math.floor(lev/3), 'CHA', -10)
    desc:add(true)
    desc:merge(num_info('AC', 10 + math.floor(lev/6),
			'to dam', 5 + math.floor((lev*2)/3),
			'speed', -5 - math.floor(lev/10)))
    desc:merge(resist_info('Resist', true, lev,
			   10, DamageType.FEAR, 25, DamageType.CONFUSION,
			   35, DamageType.NEXUS))
    desc:merge(list_info(true, lev, 30, 'free action'))
    return desc
  end,
}

newMimicryForm {
  name = 'Bear',
  desc = 'A fierce, terrible bear',
  duration = { 50, 200 },
  no_potion = true,
  on_gain = function(who, mf)
    -- Switch to Bearform combat if user requested it.
    if config.settings.tome2.auto_bear_combat then
      who:setMeleeStyle('BEARFORM_COMBAT', true)
    end
  end,
  on_lose = function(who, mf)
    -- Switch from Bearform combat to Weapon combat.
    if who.melee_style == 'BEARFORM_COMBAT' then
      who:setMeleeStyle('WEAPON_COMBAT', true)
    end
  end,
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who.speed = who.speed - 5 + math.floor(lev/5)
    who:adjustDis('to_a', 5 + math.floor((lev*2)/3), true)
    who:adjustStatForEquip('STR', math.floor(lev/11))
    who:adjustStatForEquip('INT', math.floor(lev/11))
    who:adjustStatForEquip('WIS', math.floor(lev/11))
    who:adjustStatForEquip('DEX', -1)
    who:adjustStatForEquip('CON', math.floor(lev/11))
    who:adjustStatForEquip('CHA', -10)
    if lev >= 10 then
      who.flags.FREE_ACT = true
    end
    if lev >= 20 then
      who.flags.REGENERATE = true
    end
    if lev >= 30 then
      who.resist_dam[DamageType.CONFUSION] = true
    end
    if lev >= 35 then
      who.resist_dam[DamageType.NEXUS] = true
    end
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', math.floor(lev/11),
			  'INT', math.floor(lev/11),
			  'WIS', math.floor(lev/11),
			  'DEX', -1,
			  'CON', math.floor(lev/11),
			  'CHA', -10)
    desc:add(true)
    desc:merge(num_info('AC', 5 + math.floor((lev*2)/3),
			'speed', -5 + math.floor(lev/5)))
    desc:merge(resist_info('Resist', true, lev,
			   30, DamageType.CONFUSION, 35, DamageType.NEXUS))
    desc:merge(list_info(true, 30, 10, 'free action', 20, 'regeneration'))
    return desc
  end,
}

newMimicryForm {
  name = 'Balrog',
  desc = 'A corrupted Maia',
  duration = { 30, 70 },
  no_potion = true,
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who:adjustStatForEquip('STR', 5 + math.floor(lev/5))
    who:adjustStatForEquip('INT', math.floor(lev/10))
    who:adjustStatForEquip('WIS', -5 - math.floor(lev/10))
    who:adjustStatForEquip('DEX', math.floor(lev/10))
    who:adjustStatForEquip('CON', 5 + math.floor(lev/5))
    who:adjustStatForEquip('CHA', -5 - math.floor(lev/10))
    who.immune_dam[DamageType.ACID] = true
    who.immune_dam[DamageType.FIRE] = true
    who.immune_dam[DamageType.ELEC] = true
    who.resist_dam[DamageType.DARK] = true
    who.resist_dam[DamageType.CHAOS] = true
    who.resist_dam[DamageType.POISON] = true
    who.flags.HOLD_LIFE = true
    who.flags.FEATHER = true
    who.flags.REGENERATE = true
    who.flags.SH_FIRE = true
    who.lite = who.lite + 1
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', 5 + math.floor(lev/5),
			  'INT', math.floor(lev/10),
			  'WIS', -5 - math.floor(lev/10),
			  'DEX', math.floor(lev/10),
			  'CON', 5 + math.floor(lev/5),
			  'CHA', -5 - math.floor(lev/10))
    desc:merge(resist_info('Immune to', true, lev,
			   0, DamageType.FIRE, 0, DamageType.ACID,
			   0, DamageType.ELEC))
    desc:merge(resist_info('Resist', true, lev,
			   0, DamageType.DARK, 0, DamageType.CHAOS,
			   0, DamageType.POISON))
    desc:merge(list_info(true, lev, 0, 'hold life', 0, 'feather fall',
				    0, 'regeneration', 0, 'fire aura'))
    desc:add(true)
    desc:merge(num_info('light radius', 1))
    return desc
  end,
}

newMimicryForm {
  name = 'Maia',
  desc = 'A near god-like being',
  duration = { 30, 70 },
  no_potion = true,
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who:adjustStatForEquip('STR', 5 + math.floor(lev/5))
    who:adjustStatForEquip('INT', 5 + math.floor(lev/5))
    who:adjustStatForEquip('WIS', 5 + math.floor(lev/5))
    who:adjustStatForEquip('DEX', 5 + math.floor(lev/5))
    who:adjustStatForEquip('CON', 5 + math.floor(lev/5))
    who:adjustStatForEquip('CHA', 5 + math.floor(lev/5))
    who.immune_dam[DamageType.FIRE] = true
    who.immune_dam[DamageType.ELEC] = true
    who.immune_dam[DamageType.ACID] = true
    who.immune_dam[DamageType.COLD] = true
    who.resist_dam[DamageType.POISON] = true
    who.resist_dam[DamageType.LITE] = true
    who.resist_dam[DamageType.DARK] = true
    who.resist_dam[DamageType.CHAOS] = true
    who.flags.HOLD_LIFE = true
    who.flags.FEATHER = true
    who.flags.REGENERATE = true
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', 5 + math.floor(lev/5),
			  'INT', 5 + math.floor(lev/5),
			  'WIS', 5 + math.floor(lev/5),
			  'DEX', 5 + math.floor(lev/5),
			  'CON', 5 + math.floor(lev/5),
			  'CHA', 5 + math.floor(lev/5))
    desc:merge(resist_info('Immune to', true, lev,
			   0, DamageType.FIRE, 0, DamageType.COLD,
			   0, DamageType.ACID, 0, DamageType.ELEC))
    desc:merge(resist_info('Resist', true, lev,
			   0, DamageType.POISON, 0, DamageType.LITE,
			   0, DamageType.DARK, 0, DamageType.CHAOS))
    desc:merge(list_info(true, lev, 0, 'hold life', 0, 'feather fall',
				    0, 'regeneration'))
    return desc
  end,
}

newMimicryForm {
  name = 'Fire Elem.',
  desc = 'A towering column of flames',
  duration = { 10, 10 },
  no_potion = true,
  apply = function(who, lev)
    local DamageType = require 'engine.DamageType'
    who:adjustStatForEquip('STR', 5 + math.floor(lev/5))
    who:adjustStatForEquip('DEX', 5 + math.floor(lev/5))
    who:adjustStatForEquip('WIS', -5 - math.floor(lev/5))
    who.immune_dam[DamageType.FIRE] = true
    who.resist_dam[DamageType.POISON] = true
    who.flags.SH_FIRE = true
    who.lite = who.lite + 1
  end,
  info = function(who, lev)
    local DamageType = require 'engine.DamageType'
    local desc = num_info('STR', 5 + math.floor(lev/5),
			  'DEX', 5 + math.floor(lev/5),
			  'WIS', -5 - math.floor(lev/5))
    desc:merge(resist_info('Immune to', true, lev, 0, DamageType.FIRE))
    desc:merge(resist_info('Resist', true, lev, 0, DamageType.POISON))
    desc:merge(list_info(true, lev, 0, 'fire aura'))
    desc:add(true)
    desc:merge(num_info('light radius', 1))
    return desc
  end,
}

newMimicryForm {
  name = 'Abomination',
  cloak_name = '& Abominable Cloak~',
  desc = 'Abominations are failed experiments of powerful wizards.',
  duration = { 20, 100 },
  apply = function(who, lev)
    who:adjustStatForEquip('STR', -10)
    who:adjustStatForEquip('INT', -10)
    who:adjustStatForEquip('WIS', -10)
    who:adjustStatForEquip('DEX', -10)
    who:adjustStatForEquip('CON', -10)
    who:adjustStatForEquip('CHA', -10)
    who.speed = who.speed - 10
    who.flags.AGGRAVATE = true
  end,
  info = function(who, lev)
    local desc = num_info('STR', -10, 'INT', -10, 'WIS', -10,
			  'DEX', -10, 'CON', -10, 'CHA', -10)
    desc:add(true)
    desc:merge(num_info('speed', -10))
    desc:merge(list_info(true, lev, 0, 'aggravation'))
    return desc
  end,
}
