-- $Id: options.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org


newOptionGroup { key='ui', name='User Interface' }
newOptionGroup { key='gameplay', name='Gameplay' }
newOptionGroup { key='display', name='Display' }
newOptionGroup { key='birth', name='Birth Options' }

newOption {
  key = 'linear_stats',
  name = 'Linear stats display',
  group = 'ui',
  show_in_birther = true,
  desc = [[Controls how stat values over 18 are displayed.  If enabled, stats will be formatted linearly from 18.1 through 40; if disabled, they will be shown in "percentile" format from 18/01 through 18/220 (which is shown as 18/***).]],
  default = false,
}

newOption {
  key = 'exp_display',
  name = 'Experience display',
  group = 'ui',
  default = 'Relative',
  desc = [[Controls how experience points are displayed.  '#00FFFF#Absolute#LAST#' simply displays your current experience points.  '#00FFFF#Relative#LAST#' displays the number of points needed for the next level.  '#00FFFF#Percent#LAST#' displays experience as a fraction of the difference between this level and the next level.]],
  values = { 'Absolute', 'Relative', 'Percent' },
  short_values = { ['Absolute']='Abs', ['Relative']='Rel', ['Percent']='Pct' }
}

newOption {
  key = 'food_display',
  name = 'Food display',
  group = 'ui',
  desc = [[Controls how your food state is displayed.  If disabled (the default), the usual Gorged/Full/Hungry/Weak/Fainting/Starving indicator is shown, with no indicator shown when you're between Full and Hungry; if enabled, the indicator will include your exact numerical food level, and an "OK" state between Full and Hungry.]],
  default = false,
}

newOption {
  key = 'hp_warning_pct',
  name = 'Hit point warning threshold',
  group = 'ui',
  default = 50,
  min = 0,
  max = 99,
  desc = [[Controls the 'LOW HEALTH' warning, which will appear when the player's life falls below the specified percentage of maximum.  In addition, hit points and certain other resources like mana will be displayed in red if they are below the specified percentage of maximum.  Allowed values range from 0 to 99; default is 50.]],
}

newOption {
  key = 'tiles',
  name = 'Graphics mode',
  group = 'display',
  default = 'ascii',
  require_restart = true,
  desc = [[Selects between available tile sets or ASCII.]],
  values = { 'ascii', 'ang16x16' },
  short_values = { ascii = 'ASCII', ang16x16 = 'Angband' },
  list_values = { ascii = 'ASCII tiles', ang16x16 = 'Original Angband tiles' },
}

newOption {
  key = 'ascii_tile_size',
  name = 'ASCII tile size',
  group = 'display',
  default = '12x24',
  require_restart = true,
  desc = [[Controls the tile size in ASCII graphics mode.]],
  values = { '8x16', '10x20', '12x24', '16x32', '20x40', '24x48' },
  px_sizes = { ['8x16']={8,16}, ['10x20']={10,20}, ['12x24']={12,24}, ['16x32']={16,32}, ['20x40']={20,40}, ['24x48']={24,48} },
  legacy_convert = {
    Small='8x16', Medium='10x20', Large='12x24',
    ['16x16']='8x16', ['20x20']='10x20', ['24x24']='12x24',
    ['32x32']='16x32', ['40x40']='20x40', ['48x48']='24x48',
  },
}

newOption {
  key = 'tile_size',
  name = 'Tile size',
  group = 'display',
  default = '32x32',
  require_restart = true,
  desc = [[Controls the tile size in all non-ASCII graphics modes.]],
  values = { '16x16', '32x32', '48x48', '64x64' },
  px_size = { ['16x16'] = 16, ['32x32'] = 32, ['48x48'] = 48, ['64x64'] = 64 },
}

newOption {
  key = 'hotkey_icons_size',
  name = 'Hotkey icons size',
  group = 'ui',
  default = 64,
  min = 32,
  max = 64,
  desc = [[Controls the size of icons in the hotkey bar.  Allowed values are 32 thru 64.]],
  on_change = function(_, _) game:resizeIconsHotkeysToolbar() end,
}

newOption {
  key = 'visual_hotkeys',
  name = 'Visual hotkeys feedback',
  group = 'ui',
  default = true,
  desc =  [[If enabled, activating a hotkey, whether by keystroke or mouse click, will trigger a brief visual feedback animation on the hotkey's icon.]],
}

newOption {
  key = 'ascii_player_symbols',
  name = 'Use special ASCII symbols for player',
  group = 'display',
  default = false,
  require_restart = true,
  desc = [[If enabled, the character and color used to represent your character in ASCII graphics mode will be influenced by your character's race and subrace.]],
  on_change = function(_, _)
    game.player:removeAllMOs()
    game.player:recalcAppearance()
  end,
}

newOption {
  key = 'ascii_grid_symbols',
  name = 'Modify ASCII visuals for terrain',
  group = 'display',
  default = {},
  desc = [[Allows you to modify the display of various terrain features in ASCII mode.  Symbol and color can be overridden for each terrain type.]],
  dialog = function(self, init_val)
    local function name_cb(e)
      if e.define_as == 'NOTHING' then return nil end
      return e.name
    end
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog.new('Terrain Visuals', game.zone.grid_list, init_val, name_cb)
  end,
  display_val = function(self, val)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog:displayValue(val)
  end,
  on_change = function(mods, _)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    --VisualsDialog:updateEntityList(game.zone.grid_list, mods, require('mod.class.Grid'))
    VisualsDialog:updateForEntityType('grid', mods)
    game.level.map:redisplay()
  end,
}

newOption {
  key = 'ascii_actor_symbols',
  name = 'Modify ASCII visuals for monsters',
  group = 'display',
  default = {},
  desc = [[Allows you to modify the display of various monsters in ASCII mode.  Symbol and color can be overridden for each monster type.]],
  dialog = function(self, init_val)
    local function name_cb(e)
      if e.define_as == 'PLAYER' then return nil end
      return e.name
    end
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog.new('Monster Visuals', game.zone.npc_list, init_val, name_cb)
  end,
  display_val = function(self, val)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog:displayValue(val)
  end,
  on_change = function(mods, _)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    --VisualsDialog:updateEntityList(game.zone.npc_list, mods, require('mod.class.Actor'))
    --VisualsDialog:updateLevel(false, true)
    VisualsDialog:updateForEntityType('actor', mods)
    game.level.map:redisplay()
  end,
}

newOption {
  key = 'ascii_object_symbols',
  name = 'Modify ASCII visuals for objects',
  group = 'display',
  default = {},
  desc = [[Allows you to modify the display of various objects in ASCII mode.  Symbol and color can be overridden for each object type.]],
  dialog = function(self, init_val)
    local function name_cb(e)
      -- Filter out "abstract base" entities.
      if not e.display_name and not e.name then return nil end
      if e.insta_art then return nil end
      return e:getName{no_count=true, force_id=true, unresolved=true}
    end
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog.new('Object Visuals', game.zone.object_list, init_val, name_cb)
  end,
  display_val = function(self, val)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    return VisualsDialog:displayValue(val)
  end,
  on_change = function(mods, _)
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    --VisualsDialog:updateEntityList(game.zone.object_list, mods, require('mod.class.Object'))
    --VisualsDialog:updateLevel(true, false)
    VisualsDialog:updateForEntityType('object', mods)
    game.level.map:redisplay()
  end,
}

newOption {
  key = 'faction_markers',
  name = 'Show faction markers',
  group = 'ui',
  default = { self=true, friend=true, unique=true },
  desc = [[Configure display of special markers for the player and for friendly or hostile monsters.]],
  dialog = function(self, init_val)
    local FactionsDialog = require 'mod.dialogs.FactionsDialog'
    return FactionsDialog.new('Faction Markers', 'Show faction markers for:', init_val)
  end,
  display_val = function(self, val)
    local FactionsDialog = require 'mod.dialogs.FactionsDialog'
    return FactionsDialog:displayValue(val)
  end,
  on_change = function(_, _)
    game.level.map:redisplay()
  end,
}

newOption {
  key = 'health_bars',
  name = 'Show health bars',
  group = 'ui',
  default = { self=true, friend=true },
  desc = [[Configure display of health bars for the player and for friendly or hostile monsters.]],
  dialog = function(self, init_val)
    local FactionsDialog = require 'mod.dialogs.FactionsDialog'
    return FactionsDialog.new('Health Bars', 'Show health bars for:', init_val)
  end,
  display_val = function(self, val)
    local FactionsDialog = require 'mod.dialogs.FactionsDialog'
    return FactionsDialog:displayValue(val)
  end,
  on_change = function(_, _)
    game.level.map:redisplay()
  end,
}

newOption {
  key = 'visible_antimagic_field',
  name = 'Show antimagic field',
  group = 'ui',
  default = true,
  desc = [[If enabled and the player is generating an antimagic field (by way of the Antimagic skill or anti-magic equipment), a visible indicator of the area affected by the field will be shown.]],
  on_change = function(ok, _)
    local id = game.player.EFF_ANTIMAGIC_FIELD
    local eff = game.player:hasEffect(id)
    if eff then
      local def = game.player:getEffectFromId(id)
      if ok then
	def.add_map_effect(game.player, eff)
      else
	def.remove_map_effect(game.player, eff)
      end
    end
  end,
}

newOption {
  key = 'stack_wpn_armor',
  name = 'Allow weapons and armor to stack',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, armor and weapons of the same type with identical enchantments and the same identified state will be combined into a single inventory slot.  Attempting to wield a stack of weapons or wear a stack of armor pieces will unstack one of them and wear/wield that.]],
}

newOption {
  key = 'auto_bear_combat',
  name = 'Auto-use Bearform combat',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, your melee style will automatically be changed to 'Bearform combat' every time you activate the Beorning's racial Bearform talent.]],
}

newOption {
  key = 'align_rooms',
  name = 'Generate dungeons with aligned rooms',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, the positions of rooms in generated dungeon levels will be constrained horizontally somewhat, presumably to yield nicer levels.]],
}

newOption {
  key = 'allow_small_levels',
  name = 'Allow small dungeon levels',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, dungeon levels will sometimes be smaller than usual in dungeons that allow it, typically one-third or two-thirds normal height or width.  Note that the number of generated monsters on the level is #{italic}#not#{normal}# correspondingly decreased, so smaller levels can be dangerous.  Note also that some dungeons will always have small levels, whether or not this option is enabled.]],
}

newOption {
  key = 'always_small_levels',
  name = 'Always generate small dungeon levels',
  group = 'birth',
  default = false,
  desc = [[If enabled, all dungeon levels will be smaller than usual, typically one-third or two-thirds normal height or width, except in dungeons that do not allow small levels.  This overrides the 'Allow small dungeon levels' option if set.  Note that the number of generated monsters on the level is #{italic}#not#{normal}# correspondingly decreased, so smaller levels can be dangerous.]],
}

newOption {
  key = 'ironman_rooms',
  name = 'Always generate very unusual rooms',
  group = 'birth',
  default = false,
  desc = [[If enabled, the map generator will try to place at least one vault or special room (possibly several) on each level.  Note that these rooms will frequently contain multiple powerful out-of-depth monsters, so this setting can be #{italic}#very#{normal}# dangerous.]],
}

newOption {
  key = 'allow_empty_levels',
  name = [[Allow empty 'arena' levels]],
  group = 'gameplay',
  default = true,
  desc = [[If enabled, dungeon levels will occasionally be "empty", with no wall areas between rooms.  Levels of this nature will be fully lit 80% of the time and completely dark 20% of the time.]],
}

newOption {
  key = 'npc_learn',
  name = 'Learn monster information',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, monster tooltips will only show information you have learned by observing or interacting with monsters of that type, in the style of classic T.o.M.E. 2; if disabled, monster tooltips will show all information about the monster, in the style of the ToME4 module.  Learned information is accumulated across all characters, and will continue to be accumulated even if this option is disabled.
  
(#{bold}#NOTE:#{normal}#  It should be stressed that if this option is enabled, a monster's tooltip will reflect (what you know about) a typical monster of that type (and subtype, such as Archer or Chieftain, if any), and may not fully reflect the monster itself if it has been modified by, for instance, leveling up or being hasted.  You may need to use a Rod of Probing or similar mechanism to obtain such monster-specific information.)]],
}

newOption {
  key = 'live_symbiote',
  name = [["Live" symbiotes]],
  group = 'gameplay',
  default = false,
  desc = [[If enabled, a worn symbiote will engage in semi-autonomous melee, attacking hostile monsters adjacent to the player at its own native speed.  If disabled, worn symbiotes will behave as in classic T.o.M.E. 2, adding its melee attacks to the player's at the player's speed.  In neither case will the symbiote use its native talents autonomously.]],
}

newOption {
  key = 'maximize',
  name = 'Maximize mode',
  group = 'birth',
  desc = [[Controls how race-, subrace- and class-based initial stat modifications are applied at character creation time.  Normally, a character's stats cannot be raised above 18/100 (28 in linear display) without magical equipment.  If this option is disabled, this remains true; birth modifications are only applied to initial stat values, and the stat maxima are left at 18/100.
  
If this option is enabled, on the other hand, birth modifications are applied to both initial and maximum stat values.  Thus, for instance, a Human Warrior, with +5 to STR and -2 to INT, will have a maximum STR of 18/150 and a maximum INT of 18/80 (33 resp. 26 in linear display).  You may thus want to disable this option if your choice of race, subrace and class has many net negative stat modifiers.]],
  default = true,
}

newOption {
  key = 'preserve',
  name = 'Preserve artifacts',
  group = 'birth',
  desc = [[Controls the handling of artifacts left behind on a level when you leave it:

• If disabled, artifacts left behind will be lost forever.  If there are any artifacts on the level when you enter, you will be given a special message to notify you:  "You feel there is something special about this level."

• If enabled, artifacts left behind can still be found on other levels; it is presumed that some other hapless adventurer picked the artifact up from where it was left, carried it to some other level, and lost it.  You will not receive the special message described above.  (#{bold}#NOTE:#{normal}#  Unlike the original classic T.o.M.E., this "recirculation" applies even to artifacts that have been identified, as a courtesy to Eru followers with high-level Listen to the Music spells.)]],
  default = true,
}

newOption {
  key = 'fates',
  name = 'Allow fates',
  group = 'birth',
  desc = [[If enabled, you will receive random fates, ranging from good (a fate to find a particular powerful artifact on a certain level) to mediocre (a fate to encounter some mundane monster or object on a certain level) to bad (a fate to die on a certain level).]],
  default = true,
}

newOption {
  key = 'n_quests',
  name = 'Random quests',
  group = 'birth',
  desc = [[The number of random quests that will appear in the "mainline" dungeons (Barrow Downs, Mirkwood, Mordor, Angband).  Can range from 0 (if you want no random quests) to 98 (one on every level of the mainline dungeons).]],
  default = 20,
  min = 0,
  max = 98,
}

newOption {
  key = 'floor_pickup',
  name = 'Pick things up by default',
  group = 'gameplay',
  desc = [[Controls the handling of objects on the floor of grids that you enter.  '#00FFFF#Yes#LAST#' will pick up items automatically, up to your encumbrance limit; '#00FFFF#Ask#LAST#' will prompt to pick them up, and '#00FFFF#No#LAST# will leave them on the floor.  Note that certain items, notably coins and gems, will always be picked up automatically.]],
  values = { 'Yes', 'No', 'Ask' },
  default = 'No',
}

newOption {
  key = 'disturb_detect',
  name = 'Stop running when leaving trap-detected area',
  group = 'gameplay',
  default = true,
  desc = [[If enabled, the player will stop running just before leaving an area in which traps have been detected.  This option is strongly recommended.]],
}

newOption {
  key = 'auto_quiver',
  name = 'Auto-quiver ammo fired from inventory',
  group = 'gameplay',
  default = true,
  desc = [[Affects the handling of ammo fired from inventory.  If you attempt to use the Fire Arrow, Fire Sling Shot or Fire Crossbow Bolt talents while your quiver is empty, you will be prompted to select appropriate ammo from your inventory; if this option is enabled, the ammo you select will be automatically wielded into your quiver, in addition to being fired in the usual manner.]],
}

newOption {
  key = 'auto_disarm',
  name = 'Automatically disarm traps',
  group = 'gameplay',
  default = false,
  desc = [[If this option is enabled and you attempt to move onto a grid containing a known trap, you will attempt to disarm the trap before moving onto it; if you fail, you will move onto the trap and trigger it.  If disabled (the default), known traps will block movement.  Disabling this option is recommended unless your disarming ability is particularly high.

Note that known traps always block running, independent of the setting of this option.]],
}
