-- $Id: one-ring.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

add_hooks = function()
  class:bindHook('CharacterSheet:dumpLine', function(self, data)
    local line = nil
    if game.player:isQuestStatus('one-ring', engine.Quest.FAILED) then
      line = 'You fell under the evil influence of the One Ring and decided to wear it.'
    elseif game.player:isQuestStatus('one-ring', engine.Quest.DONE) then
      line = 'You destroyed the One Ring, thus weakening Sauron.'
    end
    if line then table.insert(data.lines, line) end
  end)
end

id = 'one-ring'
name = 'The One Ring'
danger_level = 99	-- cf. src/tables.c line 4043
desc = function(self, who)
  if self:isStatus(engine.Quest.FAILED) then
    return [[You have succumbed to temptation and worn the One Ring; you are now forever corrupted.  You may still defeat Sauron and Morgoth, but in doing so you will be Arda's doom rather than its salvation.]]
  elseif self:isEnded() then
    return [[You took the One Ring to the peak of Mount Doom and destroyed it by throwing it back into the Great Fire in which it was first forged.]]
  elseif self:isStatus(engine.Quest.COMPLETED, 'found-ring') then
    return [[You have found the One Ring.  You must now take it to the peak of Mount Doom and destroy it by dropping it into the Great Fire in which it was first forged.  Once you have done this, Sauron will be weakened enough for you to permanently slay him.
But beware:  you must #{bold}#never#{normal}# use it, or you will be forever corrupted.]]
  else
    local txt = [[You cannot hope to permanently defeat Sauron while his Ring still endures.  You must find the One Ring and destroy it by taking it to the peak of Mount Doom and dropping it into the Great Fire in which it was first forged.
But beware:  you must #{bold}#never#{normal}# use it, or you will be forever corrupted.

Galadriel has suggested some places you might find the Ring:
]]
    for _, tag in ipairs(self.alt_droppers_desc) do
      local idx = self.alt_droppers_killed[tag] and 2 or 1
      local line = self.alt_droppers_desc[tag][idx]
      if idx == 2 then
	line = '#LIGHT_GREEN#' .. line .. '#LAST#'
      else
	line = line .. ('  #YELLOW##{italic}#[estimated difficulty level equivalent to dungeon level %d]#{normal}##LAST#'):format(self.alt_droppers_desc[tag][3])
      end
      txt = txt .. "\n  · " .. line
    end
    txt = txt .. [[


If you cannot reach any of these, Sauron at the base of the Pits of Angband will definitely have the Ring, though you may have to kill him several times to get it from him.]]
    return txt
  end
end

alt_droppers = {
  5,	-- count of alternate Ring-droppers
  SHELOB_SPIDER_OF_DARKNESS = 10,
  THE_WATCHER_IN_THE_WATER = 10,
  GLAURUNG_FATHER_OF_THE_DRAGONS = 10,
  FEAGWATH_THE_UNDEAD_SORCERER = 10,
  AR_PHARAZON_THE_GOLDEN = 10,
}
alt_droppers_killed = { 0 }
alt_droppers_desc = {
  -- First the Ring-droppers themselves, in increasing order of difficulty.
  'SHELOB_SPIDER_OF_DARKNESS',
  'THE_WATCHER_IN_THE_WATER',
  'AR_PHARAZON_THE_GOLDEN',
  'FEAGWATH_THE_UNDEAD_SORCERER',
  'GLAURUNG_FATHER_OF_THE_DRAGONS',
  -- Then the descriptions.
  SHELOB_SPIDER_OF_DARKNESS = {
    [[In the depths of Cirith Ungol near Mordor lurks Shelob, daughter of Ungoliant.  The Ring passed that way once; she may have waylaid its bearer.]],
    [[You have slain Shelob, Spider of Darkness; she did not have the Ring.]],
    55
  },
  THE_WATCHER_IN_THE_WATER = {
    [[Another place a Ring bearer may have fallen is a small water cave in the Dwarven mines of Moria; there is a vile many-tentacled creature there that has been the death of many.]],
    [[You have slain The Watcher in the Water; it did not have the Ring.]],
    45
  },
  GLAURUNG_FATHER_OF_THE_DRAGONS = {
    [[The father of all dragons, Glaurung, makes his nest in Erebor, the Lonely Mountain.  He has collected many treasures in his horde; the Ring may have been one of his prizes.]],
    [[You have slain Glaurung, Father of the Dragons; he did not have the Ring.]],
    70
  },
  FEAGWATH_THE_UNDEAD_SORCERER = {
    [[The unliving creatures in the Paths of the Dead are led by a mighty sorcerer, dead but kept animated by fell magics.  The Ring once passed that way, and he would certainly have coveted it.]],
    [[You have slain Feagwath, the Undead Sorcerer; he did not have the Ring.]],
    77
  },
  AR_PHARAZON_THE_GOLDEN = {
    [[Sauron may have lost his Ring while he was held captive in Númenor, in which case their last king, Ar-Pharazôn, would have claimed it.  Be careful, though, for Númenor was sunk to the depths of the ocean by the Valar for their folly; you will need some magic that allows you to breathe underwater if you mean to search there.]],
    [[You have slain Ar-Pharazôn the Golden; he did not have the Ring.]],
    55
  },
}

on_grant = function(self, who)
  -- Check all the alternate Ring-droppers to see if we've already killed
  -- any of them.
  for tag, _ in pairs(self.alt_droppers) do
    if game.uniques['mod.class.NPC/'..tag] then
      self.alt_droppers_killed[tag] = true
      self.alt_droppers_killed[1] = self.alt_droppers_killed[1] + 1
    end
  end
end

callbacks = {
  on_kill = function(who, q, target, rsrc)
    -- Only if we're the "resolved" killer.
    if rsrc then return end
    -- Only if we're looking for the Ring...
    if not q:isStatus(engine.Quest.PENDING) then return end
    -- ...and only if we haven't found it yet.
    if q:isStatus(engine.Quest.COMPLETED, 'found-ring') then return end
    -- Only if we know who we just killed. :-O
    if not target.define_as then return end
    local tag = target.define_as

    if q.alt_droppers[tag] and not q.alt_droppers_killed[tag] then
      -- Remember that we killed an alternate Ring-dropper.
      q.alt_droppers_killed[tag] = true
      q.alt_droppers_killed[1] = q.alt_droppers_killed[1] + 1
    end

    local pct = q.alt_droppers[tag]
    if tag == 'SAURON_THE_SORCERER' then
      -- 100% chance if we've killed all the alternate Ring-droppers;
      -- 30% + 10% * (# alt droppers killed) otherwise.
      if q.alt_droppers_killed[1] >= q.alt_droppers[1] then
	pct = 100
      else
	pct = 30 + 10*q.alt_droppers_killed[1]
      end
    end
    if not pct then return end

    if rng.percent(pct) then
      -- Give the player the Ring (and update the quest) after they close
      -- the dialog.
      local function cb()
	local ring = game.zone:makeEntityByName(game.level, 'object', 'OF_POWER_THE_ONE_RING')
	local dropped = nil
	-- Make sure we have room in inventory.
	while not game.player:canAddToInven('INVEN') do
	  local inven = game.player:getInven('INVEN')
	  for idx = #inven, 1, -1 do
	    if not inven[idx]:check('on_drop', game.player) then
	      dropped = inven[idx]
	      game.player:doDrop('INVEN', idx)
	      break
	    end
	  end
	end
	game.player:addObject('INVEN', ring)
	game.player:sortInven()
	game.player:setQuestStatus('one-ring', engine.Quest.COMPLETED, 'found-ring')
	if q.alt_droppers[tag] then world:gainAchievement('ONE_RING_ALT', game.player) end
      end
      local Dialog = require 'engine.ui.Dialog'
      local drop_fmt = dropped and [[, dropping your %s to make room in your inventory]] or ''
      local fmt = [[As %s falls, you notice a plain gold ring tumble from %s grasp.  You feel compelled to pick up this ring]] .. drop_fmt .. '.'
      Dialog:simpleLongPopup('Plain gold ring', fmt:format(target:getName(), target:pronoun('possessive'), (dropped and dropped:getName{no_count=true, no_mods=true, no_pval=true, no_inscr=true})), 600, cb)
    end
  end,

  on_wear_pre = function(who, q, o, inven, p)
    -- Don't bother if we haven't found the One Ring.
    if not q:isStatus(engine.Quest.COMPLETED, 'found-ring') then return end
    -- Confirm wielding the One Ring.
    if o.define_as == 'OF_POWER_THE_ONE_RING' then
      local Chat = require 'engine.Chat'
      local npc = { name=o:getName(), chat_display_entity=o }
      local chat = Chat.new('wear-ring', npc, game.player)
      local ret = who:talentDialog(ui_util.invoke_chat(chat))
      if not ret then p.no_wear = true end
    end
  end,

  on_wear = function(who, q, o, inven_id)
    if o.define_as == 'OF_POWER_THE_ONE_RING' then
      local Dialog = require 'engine.ui.Dialog'
      local text = [[The One Ring firmly binds itself to your finger as you put it on, and you feel #C_D#dark powers#LAST# sapping your soul.  Your material form weakens as you feel yourself drawn into the shadow world.]]
      local function cb()
	game.player:abandonVala('Eru')
	game.player:abandonVala('Manwe')
	game.player:abandonVala('Tulkas')
	game.player:abandonVala('Yavanna')
	-- TODO Destroy all the towns.
	game.player:grantQuest('sauron')
	world:gainAchievement('ONE_RING_WEAR', game.player)
      end
      who.ring_death_count = 0
      game.player:setQuestStatus('one-ring', engine.Quest.FAILED)
      Dialog:simpleLongPopup('One Ring', text, 600, cb)
    end
  end,

  on_drop_pre = function(who, q, o, p)
    -- Don't bother if we haven't found the One Ring.
    if not q:isStatus(engine.Quest.COMPLETED, 'found-ring') then return end
    -- Allow the One Ring to be dropped on the Great Fire.
    if o.define_as == 'OF_POWER_THE_ONE_RING' then
      local Map = require 'mod.class.Map'
      if game.level.map:checkEntity(who.x, who.y, Map.TERRAIN, 'one_ring_drop') then
	p.no_drop = false
      end
    end
  end,
}
