-- $Id: maze_vault.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Roughly mimics build_maze_vault() in src/generate.c, to build a randomly
-- generated maze vault as a subtype of the 'random_vault' room type (q.v.).
newRoom {
  id = 'maze_vault',
  no_random_gen = true,
  build_special = function(self, gen, ctr, xsize, ysize)
    -- Chance of being lit early on.
    local lit = rng.range(1, 25) > gen.dun_lev

    local dx, dy = math.floor(xsize/2) - 1, math.floor(ysize/2) - 1
    local x1, y1, x2, y2 = ctr.x - dx, ctr.y - dy, ctr.x + dx, ctr.y + dy

    -- Start with walls.
    for x = x1 - 1, x2 + 1 do
      for y = y1 - 1, y2 + 1 do
	gen.map.attrs(x, y, 'room', true)
	gen.map.attrs(x, y, 'vault', true)
	if lit then gen.map.lites(x, y, true) end
	local edge = x == x1 - 1 or x == x2 + 1 or y == y1 - 1 or y == y2 + 1
	local wall = edge and 'outer_wall' or 'inner_wall'
	gen.map(x, y, Map.TERRAIN, gen:resolve(wall, nil, false, gen.cur_lev))
      end
    end

    -- Then recursively build a spanning tree from a random root.
    local root = rng.range(0, (dx+1)*(dy+1) - 1)
    gen:r_visit(x1, y1, x2, y2, root, 0, {})

    -- Fill with monsters and treasure, low difficulty.
    gen:fill_treasure(x1, y1, x2, y2, rng.range(1, 5))
    return true
  end,
}
