-- $Id: mini_c_vault.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Roughly mimics build_mini_c_vault() in src/generate.c, to build a
-- "mini"-checkerboard vault as a subtype of the 'random_vault' room type
-- (q.v.).
newRoom {
  id = 'mini_c_vault',
  no_random_gen = true,
  build_special = function(self, gen, ctr, xsize, ysize)
    local dx, dy = math.floor(xsize/2) - 1, math.floor(ysize/2) - 1
    local x1, y1, x2, y2 = ctr.x - dx, ctr.y - dy, ctr.x + dx, ctr.y + dy

    -- First lay down permanent walls.
    for x = x1 - 1, x2 + 1 do
      for y = y1 - 1, y2 + 1 do
	gen.map(x, y, Map.TERRAIN, gen:resolve('PERMA_WALL', nil, true))
	gen.map.attrs(x, y, 'room', true)
	gen.map.attrs(x, y, 'vault', true)
      end
    end

    -- Next, reuse r_visit from the maze_vault room to build a maze.
    local root = rng.range(0, (dx+1)*(dy+1) - 1)
    gen:r_visit(x1, y1, x2, y2, root, 0, {})

    -- Finally, seal up "odd" floor grids with inner_wall to turn it into a
    -- checkerboard.
    for x = x1, x2 do
      for y = y1, y2 do
	local total = x - x1 + y - y1
	if total % 2 == 1 and gen:get_is_floor(x, y) then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, gen.cur_lev))
	end
      end
    end

    -- Add entrances.
    local xe1, ye1, xe2, ye2
    if rng.chance(2) then
      -- East/west walls.
      local y = rng.range(1, dy) + math.floor(dy/2)
      xe1, ye1, xe2, ye2 = x1 - 1, y1 + y, x2 + 1, y1 + y
    else
      -- North/south walls.
      local x = rng.range(1, dx) + math.floor(dx/2)
      xe1, ye1, xe2, ye2 = x1 + x, y1 - 1, x1 + x, y2 + 1
    end
    gen.map(xe1, ye1, Map.TERRAIN, gen:resolve('outer_wall', nil, false, gen.cur_lev))
    gen.map(xe2, ye2, Map.TERRAIN, gen:resolve('outer_wall', nil, false, gen.cur_lev))

    -- Fill with monsters and treasure, highest difficulty.
    gen:fill_treasure(x1, y1, x2, y2, 10)
    return true
  end,
}
