-- $Id: race-class.lua 2941 2017-06-20 02:16:32Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'Racial powers',
  type = 'player/racial',
  description = [[Powers conferred by the player's race]],
}

newTalentType {
  name = 'Class powers',
  type = 'player/class',
  description = [[Powers conferred by the player's class]],
}

newTalent {
  name = 'Detect Curses',
  short_name = 'PRIEST_DET_CURSE',
  display_entity = Entity.new { image='talents/detect_curses.png' },
  type = 'player/class',
  mode = 'activated',
  power = { level=7, stat='WIS', diff=14 },
  mana = 14,
  points = 0,
  action = function(self, t)
    self:inventoryApplyAll(function(inven, item, o)
      if o.cursed then o.pseudo_id = o.pseudo_id or 'cursed' end
    end)
    return true
  end,
  info = 'placeholder',
}

newTalent {
  name = 'Create Food',
  short_name = 'HOBBIT_CREATE_FOOD',
  display_entity = Entity.new { image='talents/create_food.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=15, stat='INT', diff=10 },
  mana = 10,
  points = 0,
  action = function(self, t)
    local o = game.zone:makeEntityByName(game.level, 'object', 'RATION_OF_FOOD')
    if o then
      o:identify(true, self)
      game.zone:addEntity(game.level, o, 'object', self.x, self.y)
      game.logPlayer(self, 'You cook some food.')
    end
    return true
  end,
  info = function(self, t)
    return [[Creates some food at your current location.]]
  end,
  self_knowledge_line = 'You can create food.',
}

newTalent {
  name = 'Blink',
  short_name = 'GNOME_BLINK',
  display_entity = Entity.new { image='talents/blink.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=3, stat='WIS', diff=12 },
  mana = 3,
  points = 0,
  action = function(self, t)
    spell.teleport(self, 10)
    return true
  end,
  info = function(self, t)
    return [[Teleports the player in a range of 10.]]
  end,
  self_knowledge_line = 'You can teleport yourself short distances.',
}

newTalent {
  name = 'Find Secret Passages',
  short_name = 'DWARF_PASSWALL',
  display_entity = Entity.new { image='talents/find_secret_passages.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=15, stat='DEX', diff=12 },
  mana = 15,
  points = 0,
  action = function(self, t)
    return spell.passwall(self, nil, nil, true)
  end,
  info = function(self, t)
    return [[You can find hidden ways through what looks like solid wall, allowing you to pass through walls in a cardinal direction to the other side.  You cannot stop inside a vault, in a no-teleport area or in the same grid as another monster, and you cannot pass through permanent walls.]]
  end,
  self_knowledge_line = 'You can use secret passages.',
}

newTalent {
  name = 'Remove Fear',
  short_name = 'ORC_REMOVE_FEAR',
  display_entity = Entity.new { image='talents/remove_fear.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=3, stat='WIS', diff=8 },
  mana = 5,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'You play tough.')
    self:removeEffect(self.EFF_AFRAID)
    return true
  end,
  info = function(self, t)
    return [[Removes your fear.]]
  end,
  self_knowledge_line = 'You can embolden yourself.',
}

newTalent {
  name = 'Berzerk',
  short_name = 'TROLL_BERZERK',
  display_entity = Entity.new { image='talents/berzerk.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=8, stat='STR', diff=14 },
  mana = 8,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'RAAAGH!')
    self:removeEffect(self.EFF_AFRAID)
    self:setEffect(self.EFF_SUPERHEROISM, 10 + rng.range(1, self.level), {merge='combine'})
    self:heal(30)
    return true
  end,
  info = function(self, t)
    local ret = [[Heals 30hp, cures fear, and sends you into a berzerk rage for 10+1d%d turns.  Duration will increase with your level.]]
    return ret:fmt(self.level)
  end,
  self_knowledge_line = 'You can drive yourself into a berserk frenzy.',
}

newTalent {
  name = 'Explosive Rune',
  short_name = 'HALF_OGRE_RUNE',
  display_entity = Entity.new { image='talents/explosive_rune.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=25, stat='INT', diff=15 },
  mana = 35,
  points = 0,
  action = function(self, t)
    game.log('Sorry, not implemented (HALF_OGRE_RUNE)')
  end,
  info = 'placeholder',
}

newTalent {
  name = 'Bearform',
  short_name = 'BEORNING_BEARFORM',
  display_entity = Entity.new { image='talents/bearform.png' },
  type = 'player/racial',
  mode = 'sustained',
  power = { level=2, stat='CON', diff=5 },
  mana = 5,
  points = 0,
  info = function(self, t)
    local ObjectData = require 'mod.class.interface.ObjectData'
    local mf = ObjectData.mimicry_forms['Bear']
    local lev = self:getSkillLevel('BEARFORM_COMBAT')
    local desc = tstring {
      'Transform into a bear.', true, true,
      'Effects at your current Bearform-combat skill level:', true
    }
    desc:merge(mf.info(self, lev))
    return desc:toString()
  end,
  sustain_describe = function(who, t, p)
    local ObjectData = require 'mod.class.interface.ObjectData'
    local mf = ObjectData.mimicry_forms['Bear']
    local lev = who:getSkillLevel('MIMICRY')
    local desc = tstring {
      'You are transformed into a bear, with the following effects:', true
    }
    desc:merge(mf.info(who, lev))
    return desc
  end,
  activate = function(who, def)
    game.logPlayer(who, 'You feel your body change.')
    return {}
  end,
  deactivate = function(who, def)
    game.logPlayer(who, 'You are no longer transformed.')
    return true
  end,
  apply = function(who, t)
    local ObjectData = require 'mod.class.interface.ObjectData'
    local mf = ObjectData.mimicry_forms['Bear']
    local lev = who:getSkillLevel('MIMICRY')
    mf.apply(who, lev)
  end,
}

newTalent {
  name = 'Poison Dart',
  short_name = 'KOBOLD_POISON_DART',
  display_entity = Entity.new { image='talents/poison_dart.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=12, stat='DEX', diff=14 },
  mana = 8,
  points = 0,
  range = 18,
  action = function(self, t)
    return spell.bolt_spell(self, t, DamageType.POISON, self.level)
  end,
  info = function(self, t)
    local pf = util.colorize_damtype(DamageType.POISON)
    local ret = [[Throws a poisoned dart doing %d %s damage.  Damage increases with your level.]]
    return ret:format(self.level, pf)
  end,
  self_knowledge_line = 'You can throw poisoned darts.',
}

newTalent {
  name = 'Detect Doors and Traps',
  short_name = 'PETTY_DWARF_DETECT_TRAPS',
  display_entity = Entity.new { image='talents/detect_doors_and_stairs.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=5, stat='WIS', diff=12 },
  mana = 3,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'You examine your surroundings.')
    spell.detect_traps(25, self)
    spell.detect_doors_stairs(25, self)
    return true
  end,
  info = function(self, t)
    return [[Detects doors, stairs and traps in a radius of 25.]]
  end,
  self_knowledge_line = 'You can detect hidden doors and stairs.',
}

newTalent {
  name = 'Magic Missile',
  short_name = 'DARK_ELF_MISSILE',
  display_entity = Entity.new { image='talents/magic_missile.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=2, stat='INT', diff=9 },
  mana = 2,
  points = 0,
  range = 18,
  action = function(self, t)
    local dd = 3 + math.floor((self.level - 1)/5)
    return spell.bolt_or_beam_spell(self, t, 10, DamageType.MISSILE,
				    rng.dice(dd, 4))
  end,
  info = function(self, t)
    local dd = 3 + math.floor((self.level - 1)/5)
    local mf = util.colorize_damtype(DamageType.MISSILE)
    local ret = [[Fires a magic missile from your hand, doing %dd4 %s damage.  There is a 10%% chance the missile will continue through the first monster it strikes and continue to the location you target.  Damage will increase with your level.]]
    return ret:format(dd, mf)
  end,
  self_knowledge_line = 'You can cast magic missiles.',
}

newTalent {
  name = 'Grow Trees',
  short_name = 'ENT_GROW_TREES',
  display_entity = Entity.new { image='talents/grow_trees.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=2, stat='CHA', diff=3 },
  mana = 6,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'You make the trees grow!')
    local r = math.max(1, math.floor((self.level or 0)/8))
    spell.grow_terrain(self, 'TREE', r, {'SUPPORT_GROWTH'})
    return true
  end,
  info = function(self, t)
    local r = math.max(1, math.floor((self.level or 0)/8))
    local ret = [[Quickly grow trees in random suitable grids in a radius of %d around you.  Radius will increase with player level.]]
    return ret:format(r)
  end,
}

newTalent {
  name = 'Flash Aura',
  short_name = 'ROHAN_FLASH_AURA',
  display_entity = Entity.new { image='talents/flash_aura.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=1, stat='CHA', diff=7 },
  mana = 9,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'You flash a bright aura.')
    if self.level and self.level >= 10 then
      return spell.ball_spell(self, t, DamageType.CONFUSION, (self.level or 1)*2, 2)
    else
      return spell.bolt_spell(self, t, DamageType.CONFUSION, (self.level or 1)*2)
    end
  end,
  info = function(self, t)
    local dam = (self.level or 1)*2
    local cf = util.colorize_damtype(DamageType.CONFUSION)
    local ret = [[Briefly flash a bright light aura, doing %d %s damage to a targeted opponent (damage increases with level).  At level 10, affects a radius-2 area around the target.]]
    return ret:format(cf, dam)
  end,
  -- This could go on either of the Rohan-Knight talents; we put it here.
  self_knowledge_line = 'You can use rohir powers.',
}

newTalent {
  name = 'Light-Speed Jump',
  short_name = 'ROHAN_SPEED',
  display_entity = Entity.new { image='talents/light_speed_jump.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=30, stat='WIS', diff=7 },
  mana = 30,
  points = 0,
  action = function(self, t)
    self:setEffect(self.EFF_LIGHT_SPEED, 3, {merge=combine})
    return true
  end,
  info = function(self, t)
    return [[Gives you a short boost of incredible speed, increasing your speed by 50 for 3 turns.]]
  end,
}

newTalent {
  name = 'Thunder Strike',
  short_name = 'THUNDER_STRIKE',
  display_entity = Entity.new { image='talents/thunder_strike.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=1, stat='CON', diff=6 },
  mana = function(self, t) return self.level or 1 end,
  points = 0,
  range = 18,
  action = function(self, t)
    game.logPlayer(self, 'You conjure up thunder!')
    local txy = spell.beam_spell(self, t, DamageType.ELEC, (self.level or 1)*2)
    if txy then
      spell.beam_spell(self, t, DamageType.SOUND, (self.level or 1)*2, txy.x, txy.y)
    end
    return txy and true or false
  end,
  info = function(self, t)
    local lf = util.colorize_damtype(DamageType.ELEC)
    local sf = util.colorize_damtype(DamageType.SOUND)
    local dam = (self.level or 1)*2
    local ret = [[Conjures up thunder, doing %d %s and %d %s damage in a beam.  Damage and mana cost increase with your level.]]
    return ret:format(dam, lf, dam, sf)
  end,
  -- This could go on any of the Thunderlord talents; we put it here.
  self_knowledge_line = 'You can use thunderlords powers.',
}

newTalent {
  name = 'Ride the Straight Road',
  short_name = 'THUNDER_ROAD',
  display_entity = Entity.new { image='talents/ride_the_straight_road.png' },
  type = 'player/racial',
  mode = 'activated',
  points = 0,
  power = { level=3, stat='CON', diff=6 },
  mana = 15,
  range = function(self, t) return (self.level or 0)*20 + 2 end,
  direct_hit = true,
  on_pre_use = function(who, t, silent, dry_run)
    if not spell.level_ok_teleport() then
      if not silent then
	game.logPlayer(self, 'No teleport on special levels.')
      end
      return false
    end
    return true
  end,
  use_speed = function(self, t)
    return (60 - (self.level or 0))/10
  end,
  action = function(self, t)
    game.logPlayer(self, 'You enter the straight road and fly beside the world.')
    local tg = {
      type = 'hit',
      default_target = self,
      range = self:getTalentRange(t),
      nowarning = true,
      nolock = true,
      pass_terrain = true,
      requires_knowledge = false,
    }
    local tx, ty, target = self:getTarget(tg)
    if not tx or not ty then return false end
    local ok = game.level.map:checkEntity(tx, ty, Map.TERRAIN, 'empty_bold') and not game.level.map.attrs(tx, ty, 'no_teleport')
    if not ok then
      game.logPlayer(self, 'You fail to exit correctly!')
      spell.teleport(self, 10, true)
      return { use_speed=1 }
    end
    self:move(tx, ty, true)
    return true
  end,
  info = function(self, t)
    local r = self:getTalentRange(t)
    local use_pct = 60 - (self.level or 0)
    local ret = [[Enter the straight road and travel to a selected empty grid within range %d (increases with level).  You can do this quickly (%d%% of a normal turn, decreasing with level).]]
    return ret:format(r, use_pct)
  end,
}

newTalent {
  name = 'Return to Town',
  short_name = 'THUNDER_RECALL',
  display_entity = Entity.new { image='talents/return_to_town.png' },
  type = 'player/racial',
  mode = 'activated',
  power = { level=7, stat='CON', diff=6 },
  mana = 30,
  points = 0,
  action = function(self, t)
    game.logPlayer(self, 'You enter the straight road and fly beside the world.')
    spell.set_recall(self, 1, 1)
    return true
  end,
  on_pre_use = function(who, t, silent, dry_run)
    if game.zone.town or game.zone.wilderness or game.zone.worldmap then
      if not silent then
	game.logPlayer(self, 'You cannot return to town from here.')
      end
      return false
    end
    if not game.state.recall_town then
      if not silent then
	game.logPlayer(self, "You don't know where to return to.")
      end
      return false
    end
    if not spell.level_ok_teleport() then
      if not silent then
	game.logPlayer(self, 'No recall on special levels.')
      end
      return false
    end
    return true
  end,
  info = function(self, t)
    local loc = game.state.recall_town and game.state.recall_town.name or 'none'
    local ret = [[Enter the straight road and travel back to the last town you've visited (currently %s).  Cannot be used in town (obviously), in the wilderness or on the world map.]]
    return ret:format(loc)
  end,
}

newTalent {
  name = 'Controlled Teleport',
  short_name = 'DEATHMOLD_TELEPORT',
  display_entity = Entity.new { image='talents/controlled_teleport.png' },
  type = 'player/racial',
  mode = 'activated',
  range = 18,
  points = 0,
  cooldown = function(self, t) return 100 - self.level*2 end,
  action = function(self, t)
    local tg = {
      type = 'hit',
      default_target = self,
      range = self:getTalentRange(t),
      nowarning = true,
      nolock = true,
      pass_terrain = true,
      requires_knowledge = false,
    }
    local x, y = self:getTarget(tg)
    if not x or not y then return nil end
    local fx, fy = util.findFreeGrid(x, y, 5, true, {[Map.ACTOR]=true})
    if fx and fy then self:move(fx, fy, true) end
    return true
  end,
  info = function(self, t)
    return [[Teleports you to a selected grid, which does not need to be in line of sight.  Cooldown decreases with level, down to 0 at level 50.]]
  end,
  -- This could go on any of the Deathmold talents; we put it here.
  self_knowledge_line = 'You can use the foul deathmold magic.',
}

newTalent {
  name = 'Fetch an Item',
  short_name = 'DEATHMOLD_FETCH',
  display_entity = Entity.new { image='talents/fetch_an_item.png' },
  type = 'player/racial',
  mode = 'activated',
  points = 0,
  cooldown = function(self, t) return 100 - self.level*2 end,
  action = function(self, t)
    local tg = { type='hit', range=18, talent=t, pass_terrain=true }
    local x, y = self:getTarget(tg)
    if not x or not y then return nil end
    return spell.fetch_object(self, x, y, self.level*1.5, false)
  end,
  info = function(self, t)
    local wgt = self.level*1.5
    local ret = [[Fetches an item within a range of 18 and drops it beneath you.  You cannot fetch items weighing more than %.1flb (this increases with level), and the floor beneath you must already be clear.  On the other hand, you do not need line of sight to the object.  Cooldown decreases with level, down to 0 at level 50]]
    return ret:format(ret, wgt)
  end,
}

newTalent {
  name = 'Change Level',
  short_name = 'DEATHMOLD_CHANGE_LEVEL',
  display_entity = Entity.new { image='talents/change_level.png' },
  type = 'player/racial',
  mode = 'activated',
  points = 0,
  cooldown = function(self, t) return 100 - self.level*2 end,
  action = function(self, t)
    if game.zone.flags and game.zone.flags.NO_EASY_MOVE then
      game.logPlayer(self, 'Some powerful force prevents you from teleporting.')
      return false
    end
    local function ync_cb(ok, cancel) self:talentDialogReturn(ok, cancel) end
    local up, cancel = self:talentDialog(ui_util.yesnocancelPopup('Change Level', 'Which direction?', ync_cb, 'Up', 'Down'))
    if cancel then return false end
    if up and game.level.level <= 1 then
      game.logPlayer(self, 'There is nothing above you but air.')
      return false
    end
    if not up and game.level.level >= game.zone.max_level then
      game.logPlayer(self, 'The floor is impermeable.')
      return false
    end
    game.logPlayer(self, up and 'You rise up through the ceiling.' or 'You sink through the floor.')
    game:changeLevel(game.level.level + (up and -1 or 1))
    return true
  end,
  info = function(self, t)
    return [[Teleport up or down one level (you will be prompted for direction).]]
  end,
}
