-- $Id: magic.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'Magic skill powers',
  type = 'skill/magic',
  description = 'Talents conferred by the Magic skill and subskills thereof not covered elsewhere',
}

newTalent {
  name = [[Copy Spell]],
  short_name = 'COPY_SPELL',
  display_entity = Entity.new { image='talents/copy_spell.png' },
  type = 'skill/magic',
  mode = 'activated',
  points = 0,
  action = function(self, t)
    local function filter_cb(o)
      -- ID'd, can contain a spell, and currently doesn't.
      local where = o:hasFlag('flags', 'WIELD_CAST') and 'worn_talents' or 'carried_talents'
      return o:isIDd() and o:hasFlag('flags', 'SPELL_CONTAIN') and not o[where]
    end
    local function action_cb(o, inven, floor, item)
      self:talentDialogReturn({o=o, inven=inven, floor=floor, item=item})
      return false
    end
    -- Select an object to copy onto.
    local ShowEquipInvenFloor = require 'mod.dialogs.ShowEquipInvenFloor'
    local ret = self:talentDialog(ShowEquipInvenFloor.new(t.name, self, filter_cb, action_cb))
    if not ret then return nil end
    -- Choose a spell to copy onto it.
    local UseTalents = require 'mod.dialogs.UseTalents'
    local function spell_cb(t) return t.is_spell end
    local t_id = self:talentDialog(UseTalents.new(self, nil, {no_use=true, no_bind=true, talent_dialog=true, chooser=true, filter=spell_cb}))
    if not t_id then return nil end
    -- If user selected a stack of objects, pull off one of them to copy onto.
    local o, unstack = ret.o, false
    if o:getNumber() > 1 then
      o = self:getOneObject(ret.inven, ret.item, ret.floor, {vocal_left=true})
      unstack = true
    end
    -- Actually copy the spell onto it.
    local where = o:hasFlag('flags', 'WIELD_CAST') and 'worn_talents' or 'carried_talents'
    o[where] = { t_id }
    game.logPlayer(self, 'You create:  %s', o:getName{do_color=true})
    -- If we unstacked this, put it back into inventory.
    if unstack then self:addOrDrop(self.INVEN_INVEN, o, nil, true) end
    -- Shouldn't need to update player talents, but just in case...
    self:sortInven()
    self:recalcEverything()
    -- Done.
    return true
  end,
  info = [[Inscribes a spell from one of your spellbooks onto a suitable item, allowing you to cast the spell from the item without needing the spellbook.  Typically such items will need to be worn in order to cast the inscribed spell.]],
}

newTalent {
  name = [[Cast Runespell]],
  short_name = 'RUNECRAFT',
  display_entity = Entity.new { image='talents/cast_runespell.png' },
  type = 'skill/magic',
  mode = 'activated',
  points = 0,
  action = function(self, t)
    game.log('Sorry, not implemented (RUNECRAFT)')
  end,
  info = [[Watch this space]],
}
