-- $Id: AttackType.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local DamageType = require 'engine.DamageType'

module(..., package.seeall, class.make)

_M.atk_type_def = {}
_M.atk_method_def = {}

function _M:loadDefinition(file)
  local f, err = loadfile(file)
  if not f and err then error(err) end
  setfenv(f, setmetatable({
    AttackType = _M,
    newAttackType = function(t) self:newAttackType(t) end,
  }, {__index=_G}))
  f()
end

function _M:newAttackType(t)
  assert(t.id, 'no attack type id')
  t.power = t.power or 0
  table.insert(self.atk_type_def, t)
  self[t.id] = #self.atk_type_def
end

function _M:get(id)
  assert(self.atk_type_def[id], 'attack type '..tostring(id)..' used but not defined')
  return self.atk_type_def[id]
end

-- We split this out so that Player can use this for monster-style attacks
-- for Possessors (TODO).
function _M:monAttackTarget(src, target, p)
  local type = p.type or DamageType.PHYSICAL
  local dt = DamageType:get(type)
  local mthd = self:get(p.method)

  if not dt or src:checkHit(target, dt.power) then
    local dam = table.clone(p, true)
    dam[1] = p.dam and rng.dice(p.dam[1], p.dam[2]) or 0
    if not src.player and not src.symbiosis and not target.player then
      -- This is described as "Hack need more punch against monsters"
      -- (cf. src/melee2.c line 5889)
      dam[1] = dam[1] * 3
    end
    dam.weapon, dam.atk_msg, dam.short_0dam = p, mthd.msg, not p.dam
    local ret_dam, learn = dt.projector(src, target.x, target.y, type, dam)
    if not src.dead and not target.dead then
      if src.on_melee_hit then src:on_melee_hit(target, p) end
      src:fireCallbacks('on_melee_blow', target, true, p)
      if not mthd.no_retal and target.onHitInMelee then
	target:onHitInMelee(src, p)
      end
    end
    -- Slight Hack(TM):  Attacks with no damage type are always considered
    -- "obvious" [cf. src/melee1.c lines 1872-1881].
    learn = learn or not p.type
    return true, ret_dam, learn
  end
  if not mthd.silent_miss then
    game.logSeen(target, '%s misses %s.', src.name, target.name)
  end
  src:fireCallbacks('on_melee_blow', target, false, p)
  -- Slight Hack(TM), redux:  Attacks with no damage type are always
  -- considered "obvious", even if they miss. [cf. src/melee1.c ibid.].
  local learn = not p.type
  return false, 0, learn
end
