-- $Id: LogFlasher.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local LogFlasher = require 'engine.LogFlasher'

module(..., package.seeall, class.inherit(LogFlasher))

function _M:init(x, y, w, h, max, fontname, fontsize, color, bgcolor)
  LogFlasher.init(self, x, y, w, h, max, fontname, fontsize, color, bgcolor)
  self.last_msg = nil
  self.last_msg_rpt = 0
  getmetatable(self).__call = _M.call
end

-- Overloaded to manage a (x2) etc. suffix for repeated messages.
function _M:call(style, str, ...)
  if self.flashing == 0 and #self.log > 0 then
    self.log = {}
    self.last_msg = nil
    self.last_msg_rpt = 0
  end

  local base = ""
  if #self.log > 0 then base = table.remove(self.log) end

  str = str:format(...)
  if str == self.last_msg then
    local old_sfx = ('%%s*%%(×%d%%)$'):format(self.last_msg_rpt)
    base = base:gsub(old_sfx, '', 1)
    self.last_msg_rpt = self.last_msg_rpt + 1
    str = ('(×%d)'):format(self.last_msg_rpt)
  else
    self.last_msg = str
    self.last_msg_rpt = 1
  end

  local lines = (base .. " " .. str):splitLines(self.w - 4, self.font)
  for i = 1, #lines do
    table.insert(self.log, lines[i])
  end
  self.flashing_style = style
  self.flashing = 20

  self:getNext()
end

-- Overloaded to actually roll the flasher past the first line as needed.
function _M:toScreen(nb_keyframe)
  nb_keyframe = nb_keyframe or 1
  self.changed = false

  -- Erase and the display the map
  local bg = table.clone(self.bgcolor)
  local st = self.flashing_style
  local idx = st == LogFlasher.BAD and 1 or st == LogFlasher.GOOD and 2 or 3
  bg[idx] = bg[idx] + self.flashing*10

  core.display.drawQuad(self.display_x, self.display_y, self.w, self.h, bg[1], bg[2], bg[3], 255)
  self.texture:toScreenFull(self.display_x, self.display_y, self.w, self.h, self.texture_w, self.texture_h)

  if self.flashing > 0 then
    self.flashing = self.flashing - nb_keyframe
  elseif #self.log > 0 then
    self:getNext(true)
  end
end
