-- $Id: ChangeZoneDialog.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local List = require 'engine.ui.List'
local GetQuantity = require 'engine.dialogs.GetQuantity'

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
  self:generateList()
  Dialog.init(self, 'Change Zone', 1, 1)

  self.c_list = List.new {
    width = 400,
    height = 500,
    scrollbar = true,
    list = self.list,
    fct = function(item) self:use(item) end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_list },
  }
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

function _M:use(item)
  if not item then return end
  game:unregisterDialog(self)

  if not item.min or item.min == item.max then
    game:changeLevel(1, item.zone)
  else
    local function cb(qty)
      game:changeLevel(qty - item.min + 1, item.zone)
    end
    game:registerDialog(GetQuantity.new('Zone: '..item.name, ('Level %d-%d'):format(item.min, item.max), item.min, item.max, cb, min))
  end
end

function _M:generateList()
  local list = {}

  -- Overkill for now, but I suppose we might eventually have addons that
  -- add new zones...
  local function parse(base, add, add_simple)
    for _, dir in ipairs(fs.list(base.."/zones/")) do
      local f = loadfile(base.."/zones/"..dir.."/zone.lua")
      if f then
	setfenv(f, setmetatable({}, {__index=_G}))
	local ok, z = pcall(f)
	if ok then
	  list[#list+1] = {
	    name = z.name..(add_simple and " ["..add_simple.."]" or ""),
	    zone = add..dir,
	    min = z.level_range and z.level_range[1],
	    max = z.level_range and z.level_range[2],
	  }
	end
      end
    end
  end

  parse('/data', '')
  for _, dir in ipairs(fs.list("/")) do
    local _, _, addon = dir:find("^data%-(.+)$")
    if addon then parse("/"..dir, addon.."+", addon) end
  end

  table.sort(list, function(a, b) return a.name < b.name end)

  local chars = {}
  for i, v in ipairs(list) do
    v.name = v.name
    chars[self:makeKeyChar(i)] = i
  end
  list.chars = chars

  self.list = list
end
