-- $Id: trap-util.lua 2837 2017-04-11 23:37:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Assorted trap-related utility functions.

trap_util = {}
trap_util.default_filter = { deep_flags={flags={FLOOR=true}} }
trap_util.chest_filter = { deep_flags={flags={CHEST=true}} }

function trap_util.generator()
  return game.zone:getGenerator('trap', game.level, game.level.data.generator.trap)
end

function trap_util.stat_trap(trap, x, y, who)
  local p = trap.params or {}
  local drain, sust = who:bumpStatDown(trap.stat, p)
  return true, false, (drain or sust)
end

function trap_util.curse_weapon_trap(who)
  local inv = who:getInven('MAIN_HAND')
  if not inv or not inv[1] then return true, false, false end
  local wpn = inv[1]
  local wpn_name = wpn:getName{ no_count=true }
  if wpn:isArtifact() and rng.percent(50) then
    game.logPlayer(who, 'A terrible black aura tries to surround your %s, but it resists the effects!', wpn_name)
  else
    game.logPlayer(who, 'A terrible black aura blasts your %s!', wpn_name)
    wpn.unique = nil
    wpn.artifact_name = nil
    wpn.keep_base = nil
    wpn.ego_names = { prefix='SHATTERED' }
    wpn.to_h = -rng.dice(2, 5)
    wpn.to_d = -rng.dice(2, 5)
    wpn.armor = 0
    wpn.to_a = 0
    wpn.combat = { 0, 0 }
    wpn.cursed = true
    -- Uninstall callbacks before we remove them.
    who:uninstallCallbacks(wpn, 'worn_callbacks', wpn)
    who:uninstallCallbacks(wpn, 'carried_callbacks', wpn)
    for _, f in ({'convey_flags', 'esp_flags', 'stat_flags', 'pval_flags',
		  'cum_flags', 'slay_flags', 'convey_resist_dam',
		  'convey_immune_dam', 'convey_melee_retal', 'brand_dam',
		  'convey_flags', 'flags', 'ignore_dam', 'worn_callbacks',
		  'carried_callbacks'}) do
      wpn[f] = nil
    end

    who.changed = true
    if who.recalcEverything then who:recalcEverything() end
  end
  return true, false, true
end

function trap_util.curse_armor_trap(who)
  local inv = who:getInven('BODY')
  if not inv or not inv[1] then return false, false end
  local armor = inv[1]
  local armor_name = armor:getName{ no_count=true }
  if armor:isArtifact() and rng.percent(50) then
    game.logPlayer(who, 'A terrible black aura tries to surround your %s, but it resists the effects!', armor_name)
  else
    game.logPlayer(who, 'A terrible black aura blasts your %s!', armor_name)
    armor.unique = nil
    armor.artifact_name = nil
    armor.keep_base = nil
    armor.ego_names = { prefix='BLASTED' }
    armor.to_a = -rng.dice(2, 5)
    armor.to_h = 0
    armor.to_d = 0
    armor.armor = 0
    armor.combat = { 0, 0 }
    armor.cursed = true
    -- Uninstall callbacks before we remove them.
    who:uninstallCallbacks(armor, 'worn_callbacks', armor)
    who:uninstallCallbacks(armor, 'carried_callbacks', armor)
    for _, f in ({'convey_flags', 'esp_flags', 'stat_flags', 'pval_flags',
		  'cum_flags', 'slay_flags', 'convey_resist_dam',
		  'convey_immune_dam', 'convey_melee_retal', 'brand_dam',
		  'convey_flags', 'flags', 'ignore_dam', 'worn_callbacks',
		  'carried_callbacks'}) do
      armor[f] = nil
    end

    who.changed = true
    if who.recalcEverything then who:recalcEverything() end
  end
  return true, false, true
end

function trap_util.summon_trap(trap, x, y, who)
  if trap.msg then game.logPlayer(who, trap.msg) end
  local ret = spell.summon(self, x, y, trap.filter or {}, rng.range(1, 3))
  return true, false, (ret and true or false)
end

-- cf. player_handle_missile_trap() in src/traps.c
function trap_util.missile_trap(trap, x, y, who)
  local DamageType = require 'engine.DamageType'
  local p = trap.params
  local num = p.multi and 2 + math.floor(game.zone.level_range[2]/15) or 1
  local o = game.zone:makeEntityByName(game.level, 'object', p.name)
  if not o then return true, false, false end
  o:setNumber(num)
  game.logPlayer(who, 'Suddemly %s hit%s you!', o:getName(), (num == 1 and '' or 's'))
  for _ = 1, num do
    who:takeHit(rng.dice(p.dam[1], p.dam[2]), trap)
    if p.poison and not who:resistDam(DamageType.POISON, {or_immune=true}) then
      local dur = p.poison[1] + rng.range(1, p.poison[2])
      who:setEffect(who.EFF_POISON, dur, { merge='combine' })
    end
  end
  local dx, dy = util.drop_loc_near(who.x, who.y, 5)
  if dx and dy then game.level.map:addObject(dx, dy, o) end
  return true, false, true
end

-- cf. player_handle_breath_trap() in src/traps.c
function trap_util.breath_trap(trap, x, y, who)
  local p = trap.params
  -- Traps get deadlier as we go deeper.
  local adj = math.floor(game.zone.level_range[2]/15)
  local dd, ds = trap.damage[1] + adj, trap.damage[2] + adj
  -- Slight Hack(TM) to pass trap location to spell.ball_spell() [q.v.].
  local dam = { rng.dice(dd, ds), start_x=x, start_y=y }
  spell.ball_spell(trap, nil, p.dt, dam, p.r, who.x, who.y, 1)
  return true, false, true
end

function trap_util.piety_trap(trap, x, y, who)
  local p = trap.params
  local god, ok = table.get(who, 'descriptor', 'god')
  if not ok or not god or god == 'Nobody' then
    game.logPlayer(who, "Suddenly you feel glad you're an atheist.")
  else
    local hit = (p.hit or 0) + (p.hit_lev or 0)*(who.level or 1)
    game.logPlayer(who, p.msg, god)
    who:incPiety(-hit)
  end
  return true, false, false
end

function trap_util.teleport_item(o, who)
  if not o or o.no_obj_teleport then return false end

  local Map = require 'mod.class.Map'
  local map = game.level.map
  local tx, ty = nil, nil
  local ident = false
  repeat
    tx, ty = rng.range(0, map.w-1), rng.range(0, map.h-1)
    if map:checkEntity(tx, ty, Map.TERRAIN, 'floor') then
      tx, ty = util.drop_loc_near(tx, ty, 3)
    else
      tx, ty = nil, nil
    end
  until tx and ty

  local name = o:getName{no_count=true}
  local is = o:getNumber() == 1 and 'is' or 'are'
  if who:has('BLIND') then
    game.logPlayer(who, 'You hear something move.')
  elseif map.seens(x, y) then
    game.logPlayer(who, 'The %s %s suddenly elsewhere.', name, is)
    ident = true
  else
    game.logPlayer(who, "You suddently don't see the %s anymore!", name)
    ident = true
  end

  local ok, idx = game.level.map:addObject(tx, ty, o)
  if ok and map.attrs(tx, ty, 'on_drop') then
    map.attrs(tx, ty, 'on_drop')(who, tx, ty, o)
  end

  return ident
end

-- cf. player_activate_trap_type() in src/traps.c, specifically the
-- switches for TRAP_OF_DROP_ITEMS et al.
function trap_util.drop_item_trap(trap, x, y, who)
  local inven = who:getInven('INVEN')
  if not inven or #inven == 0 then
    game.logPlayer(who, 'You hear a sudden, strange sound.')
    return true, false, false
  end
  -- Walk inventory backward to avoid index shifting
  local do_msg, ident = false, false
  for i = #inven, 1, -1 do
    if rng.percent(trap.drop_prob) and not inven[i]:hasFlag('flags', 'PERMA_CURSE') and not inven[i].no_obj_teleport then
      who:dropFloor(inven, i, false, true)
      if not do_msg then
	game.logPlayer(who, trap.drop_msg)
	do_msg = true
      end
      ident = true
    end
  end
  if not ident then
    game.logPlayer(who, 'You hear a sudden, strange sound.')
  end
  return true, false, ident
end

-- cf. player_activate_trap_of_walls() in src/traps.c
function trap_util.walls_trap(trap, x, y, who)
  local Map = require 'mod.class.Map'
  local map = game.level.map
  local hit = {}
  local lev = game.zone:level_adjust_level(game.level)

  for xx = who.x - 2, who.x + 2 do
    for yy = who.y - 2, who.y + 2 do
      if map:isBound(xx, yy) and not map(xx, yy, Map.ACTOR) then
	-- Unlight, forget, and remove room/vault markers.
	map:unattr(xx, yy, 'room')
	map:unattr(xx, yy, 'vault')
	map.lites(xx, yy, false)
	map.remembers(xx, yy, false)
	if (xx ~= who.x or yy ~= who.y) and rng.percent(10 + lev) then
	  hit[#hit+1] = {x=xx, y=yy}
	end
	map:updateMap(xx, yy)
      end
    end
  end

  for _, xy in ipairs(hit) do
    -- Now, this bit confuses me:  In the previous loop, the T2 code makes
    -- sure there isn't a monster in a grid before marking it to change,
    -- and we reproduce this.  In this loop, the T2 code checks each marked
    -- grid for a monster and tries to move it somewhere safe, as in the
    -- earthquake() code --- but there shouldn't *be* a monster in a marked
    -- gridd, because we checked for that in the first loop.  We'll just
    -- punt and skip any grid with an actor in it.
    if not map(xy.x, xy.y, Map.ACTOR) and map:checkEntity(xy.x, xy.y, Map.TERRAIN, 'valid_bold') then
      -- Destroy all objects on the grid.  The previous :valid_bold() check
      -- ensures that there are no artifacts on the grid.
      while map:getObjectTotal(xy.x, xy.y) > 0 do
	local o = map:getObject(xy.x, xy.y, 1)
	map:removeObject(xy.x, xy.y, 1)
	game.level:removeEntity(o)
	if o.removed then o:removed() end
      end
      -- Change floor to wall and vice versa
      local floor = map:checkEntity(xy.x, xy.y, Map.TERRAIN, 'floor')
      local new_grid = floor and spell.wall_grid() or spell.floor_grid()
      if new_grid then
	game.level.map(xy.x, xy.y, Map.TERRAIN, new_grid)
	new_grid:check('addedToLevel', game.level, xy.x, xy.y)
	new_grid:check('on_added', game.level, xy.x, xy.y)
	map:updateMap(xy.x, xy.y)
      end
    end
  end

  spell.clobber_FOVs(who.x, who.y)
  game.logPlayer(who, 'Suddenly the cave shifts around you.  The air is getting stale!')
  return true, false, true
end

-- Set a trap.  Must be called in a coroutine context.  We assume that
-- caller has checked prerequisites [cf. SET_TRAP talent's on_pre_use()].
function trap_util.set_trap(who, t)
  local Map = require 'mod.class.Map'
  local ShowInventory = require 'mod.dialogs.ShowInventory'
  local inven = who:getInven('INVEN')
  -- Select the trapkit.
  local function sel_cb(o, _) who:talentDialogReturn(o) end
  local function kit_filter(o)
    return o.type == 'misc' and o.subtype == 'trapkit'
  end
  local tk = who:talentDialog(ShowInventory.new('Select trap kit', inven, kit_filter, sel_cb, who))
  if not tk then return false end
  -- Select payload for trapkit.
  if not tk.payload_filter then
    game.logPlayer(who, '?? This trap kit appears to be malformed.')
    return false
  end
  if not util.findInInventoryBy(inven, tk.payload_filter) then
    game.logPlayer(who, 'You have nothing to load this trap with.')
    return false
  end
  local pl = who:talentDialog(ShowInventory.new('Select trap payload', inven, tk.payload_filter, sel_cb, who))
  if not pl then return false end
  -- For non-wand/staff/rod payload, select number.
  local num = 1
  if not tk.no_multi_payload then
    num = tk:num_shots()
    if tk:hasAnyFlag('flags', 'AUTOMATIC_5', 'AUTOMATIC_99') then
      num = 99
    end
    num = math.min(num, pl:getNumber())
    local prompt = ('(1-%d)'):format(num)
    local function n_cb(qty) who:talentDialogReturn(qty) end
    num = who:talentDialog(ui_util.get_quantity('How many to load?', prompt, num, num, n_cb))
    if not num or num <= 0 then return false end
  end
  -- Pull the required items from inventory.
  local item
  tk, item = who:findInInventoryByObject(inven, tk)
  if not tk or not item then
    game.log('??? where did the trap kit go?')
    return false
  end
  tk, _ = who:removeObject(inven, item, 1)
  pl, item = who:findInInventoryByObject(inven, pl)
  if not pl or not item then
    game.log('??? where did the payload go?')
    -- Put the trapkit back if we couldn't find the payload.
    who:addOrDropObject(inven, tk, nil, nil, {vocal_drop=true})
    return false
  end
  pl, _ = who:removeObject(inven, item, num)
  who:sortInven()
  -- Build the trap entity.
  local Trap = require 'mod.class.Trap'
  local tr = Trap.new {
    define_as = 'PLAYER_TRAP',
    name = [[Player Trap (how are you seeing this?)]],
    idx = 0,
    display = '^',
    color = colors.C_v,
    image = 'terrain/monster_trap.png',
    difficulty = 0,
    probability = 0,
    min_level = 0,
    flags = { FLOOR=true },
    -- For use by Trap:canTrigger()
    faction = 'players',
    -- Sneaky Hack(TM):  If we're careful, we can use this trap as the
    -- projection source for the ARCHERY damage type.
    to_h = 0,
    to_h_ranged = 20,
    to_d_ranged = 0,
    skill_archery = 0,
    extra_might = 0,
    -- cf. Trap:resolveSource()
    setter = who,
    player_trap = true,
    trapkit = tk,
    payload = pl,
    triggered = tk.triggered,
    on_disarm = function(self, x, y, who)
      -- Leave the trapkit and remaining payload on the floor.
      game.level.map:addObject(x, y, self.trapkit)
      if self.payload then game.level.map:addObject(x, y, self.payload) end
    end,
  }
  tr:setKnown(who, true)
  tr:identify(true)
  -- Set the trap.
  game.level.map(who.x, who.y, Map.TRAP, tr)
end

-- Utility function used by the triggered() methods below.
function trap_util.apply_trap_spec(trap, x, y, who, spec)
  if spec.dam then
    local dam = type(spec.dam) == 'table' and rng.dice(spec.dam[1], spec.dam[2]) or spec.dam
    -- Slight Hack(TM) to pass trap location to spell.ball_spell() [q.v.].
    spell.ball_spell(trap, nil, spec.damtype, {dam, no_grid=true, start_x=x, start_y=y}, spec.r, x, y, 0)
  end
  if spec.side_effect then spec:side_effect(trap, x, y, who) end
end

-- triggered() method for Device Trap Sets.
-- cf. mon_hit_trap() in src/traps.c
function trap_util.trigger_device_trap(trap, x, y, who)
  local stick = trap.payload
  if not stick then return false, false, false end
  local spec, ok, max_shots
  if stick.subtype == 'rod' then
    spec, ok = table.get(stick, 'rod_tip', 'in_trap')
    max_shots = math.floor(stick.power / stick.use_power.power)
  else
    spec, ok = stick.in_trap, true
    max_shots = e.power
  end
  if not ok or not spec then return false, false, false end
  -- Fire at will (or until the target is dead)
  local shots = trap:numShots(max_shots)
  for _ = 1, shots do
    if who.dead then break end
    trap_util.apply_trap_spec(trap, x, y, who, spec)
    stick.power = stick.power - (stick.subtype == 'rod' and stick.use_power.power or 1)
  end
  trap_util.auto_disarm_player_trap_on_use(trap, x, y, who)
  return true, false, false
end

-- triggered() method for Fumes Trap Sets and Magic Trap Sets.
-- cf. mon_hit_trap() in src/traps.c
function trap_util.trigger_consumables_trap(trap, x, y, who)
  local stuff = trap.payload
  local n_stuff = stuff and stuff:getNumber() or 0
  if n_stuff <= 0 or not stuff.in_trap then return false, false, false end
  -- Fire at will (or until the target is dead)
  local shots = trap:numShots(n_stuff)
  local name = who:getName{def_art=true}
  for _ = 1, shots do
    if who.dead then break end
    if trap.trigger_msg then
      game.logSeen(who, trap.trigger_msg:gsub('#Target#', name:capitalize()):gsub('#target#', name))
    end
    trap_util.apply_trap_spec(trap, x, y, who, stuff.in_trap)
    local uo, last = trap.payload:unstack(1)
    if game.level:hasEntity(uo) then game.level:removeEntity(uo) end
    if game:hasEntity(uo) then game:removeEntity(uo) end
    if uo.removed then uo:removed() end
    if last then
      trap.payload = nil
      break
    end
  end
  trap_util.auto_disarm_player_trap_on_use(trap, x, y, who)
  return true, false, false
end

-- triggered() method for Arrow/Bolt/Catapult Trap Sets.
-- cf. mon_hit_trap() in src/traps.c
function trap_util.trigger_ammo_trap(trap, x, y, who)
  local DamageType = require 'engine.DamageType'
  local ammo = trap.payload
  local n_ammo = ammo and ammo:getNumber() or 0
  if n_ammo <= 0 then return false, false, false end
  -- Fire at will (or until the target is dead)
  local shots = trap:numShots(n_ammo)
  for _ = 1, shots do
    if who.dead then break end
    local shot, last = ammo:unstack(1)
    -- Questionable Hack(TM):  We try to re-use the archery infrastructure.
    local tg = { type='hit', range=1, start_x=x, start_y=y }	-- ??
    local p = { 0, ammo=shot, launcher=trap.trapkit }
    trap:project(tg, x, y, DamageType.ARCHERY, p)
    if last then
      trap.payload = nil
      break
    end
  end
  trap_util.auto_disarm_player_trap_on_use(trap, x, y, who)
  return true, false, false
end

-- Disarm player trap after use, except for Fully Automatic traps or
-- Automatic traps (20% chance of disarming)
function trap_util.auto_disarm_player_trap_on_use(trap, x, y, who)
  if not trap.trapkit or not trap.trapkit:auto_reset() then
    trap:disarm(x, y, who, {force=true, silent=true})
  end
end
